@@ -6,100 +6,141 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
  * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc 21904 2011-01-21 00:04:16Z fyodor $ */
+/* $Id: Target.cc 37126 2018-01-28 21:18:17Z fyodor $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
 #endif
 
 #include "Target.h"
+#include "FingerPrintResults.h" /* for ~FingerPrintResults() */
 #include <dnet.h>
 #include "nbase.h"
 #include "NmapOps.h"
-#include "utils.h"
+#include "nmap.h"
 #include "nmap_error.h"
 
 extern NmapOps o;
@@ -113,6 +154,7 @@ void Target::Initialize() {
   targetname = NULL;
   memset(&seq, 0, sizeof(seq));
   distance = -1;
+  distance_calculation_method = DIST_METHOD_NONE;
   FPR = NULL;
   osscan_flag = OS_NOTPERF;
   weird_responses = flags = 0;
@@ -124,6 +166,7 @@ void Target::Initialize() {
   targetsocklen = sourcesocklen = nexthopsocklen = 0;
   directly_connected = -1;
   targetipstring[0] = '\0';
+  sourceipstring[0] = '\0';
   nameIPBuf = NULL;
   memset(&MACaddress, 0, sizeof(MACaddress));
   memset(&SrcMACaddress, 0, sizeof(SrcMACaddress));
@@ -143,11 +186,11 @@ void Target::Initialize() {
 
 
 const char * Target::deviceName() const {
-	return (devname[0] != '\0')? devname : NULL;
+        return (devname[0] != '\0')? devname : NULL;
 }
 
 const char * Target::deviceFullName() const {
-	return (devfullname[0] != '\0')? devfullname : NULL; 
+        return (devfullname[0] != '\0')? devfullname : NULL;
 }
 
 void Target::Recycle() {
@@ -157,6 +200,12 @@ void Target::Recycle() {
 
 Target::~Target() {
   FreeInternal();
+#ifndef NOLUA
+  while (!scriptResults.empty()) {
+    scriptResults.front().clear();
+    scriptResults.pop_front();
+  }
+#endif
 }
 
 void Target::FreeInternal() {
@@ -177,32 +226,57 @@ void Target::FreeInternal() {
 
 /*  Creates a "presentation" formatted string out of the IPv4/IPv6 address.
     Called when the IP changes */
-void Target::GenerateIPString() {
+void Target::GenerateTargetIPString() {
   struct sockaddr_in *sin = (struct sockaddr_in *) &targetsock;
 #if HAVE_IPV6
   struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &targetsock;
 #endif
 
-  if (inet_ntop(sin->sin_family, (sin->sin_family == AF_INET)? 
-                (char *) &sin->sin_addr : 
+  if (inet_ntop(sin->sin_family, (sin->sin_family == AF_INET)?
+                (char *) &sin->sin_addr :
 #if HAVE_IPV6
-                (char *) &sin6->sin6_addr, 
+                (char *) &sin6->sin6_addr,
 #else
                 (char *) NULL,
 #endif
-		targetipstring, sizeof(targetipstring)) == NULL) {
+                targetipstring, sizeof(targetipstring)) == NULL) {
     fatal("Failed to convert target address to presentation format!?!  Error: %s", strerror(socket_errno()));
   }
 }
 
+/*  Creates a "presentation" formatted string out of the IPv4/IPv6 address.
+    Called when the IP changes */
+void Target::GenerateSourceIPString() {
+  struct sockaddr_in *sin = (struct sockaddr_in *) &sourcesock;
+#if HAVE_IPV6
+  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &sourcesock;
+#endif
+
+  if (inet_ntop(sin->sin_family, (sin->sin_family == AF_INET)?
+                (char *) &sin->sin_addr :
+#if HAVE_IPV6
+                (char *) &sin6->sin6_addr,
+#else
+                (char *) NULL,
+#endif
+                sourceipstring, sizeof(sourceipstring)) == NULL) {
+    fatal("Failed to convert source address to presentation format!?!  Error: %s", strerror(socket_errno()));
+  }
+}
+
+/* Returns the address family of the destination address. */
+int Target::af() const {
+  return targetsock.ss_family;
+}
+
 /* Fills a sockaddr_storage with the AF_INET or AF_INET6 address
      information of the target.  This is a preferred way to get the
      address since it is portable for IPv6 hosts.  Returns 0 for
      success. ss_len must be provided.  It is not examined, but is set
      to the size of the sockaddr copied in. */
-int Target::TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
+int Target::TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len) const {
   assert(ss);
-  assert(ss_len);  
+  assert(ss_len);
   if (targetsocklen <= 0)
     return 1;
   assert(targetsocklen <= sizeof(*ss));
@@ -211,9 +285,13 @@ int Target::TargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
   return 0;
 }
 
+const struct sockaddr_storage *Target::TargetSockAddr() const {
+  return &targetsock;
+}
+
 /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
-void Target::setTargetSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
+void Target::setTargetSockAddr(const struct sockaddr_storage *ss, size_t ss_len) {
 
   assert(ss_len > 0 && ss_len <= sizeof(*ss));
   if (targetsocklen > 0) {
@@ -224,7 +302,7 @@ void Target::setTargetSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
   }
   memcpy(&targetsock, ss, ss_len);
   targetsocklen = ss_len;
-  GenerateIPString();
+  GenerateTargetIPString();
   /* The ports array needs to know a name too */
   ports.setIdStr(targetipstr());
 }
@@ -247,8 +325,16 @@ const struct in_addr *Target::v4hostip() const {
   return NULL;
 }
 
+const struct in6_addr *Target::v6hostip() const {
+  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &targetsock;
+  if (sin6->sin6_family == AF_INET6) {
+    return &(sin6->sin6_addr);
+  }
+  return NULL;
+}
+
  /* The source address used to reach the target */
-int Target::SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
+int Target::SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) const {
   if (sourcesocklen <= 0)
     return 1;
   assert(sourcesocklen <= sizeof(*ss));
@@ -259,21 +345,22 @@ int Target::SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
   return 0;
 }
 
+const struct sockaddr_storage *Target::SourceSockAddr() const {
+  return &sourcesock;
+}
+
 /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
-void Target::setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
+void Target::setSourceSockAddr(const struct sockaddr_storage *ss, size_t ss_len) {
   assert(ss_len > 0 && ss_len <= sizeof(*ss));
   memcpy(&sourcesock, ss, ss_len);
   sourcesocklen = ss_len;
+  GenerateSourceIPString();
 }
 
 // Returns IPv4 host address or {0} if unavailable.
-struct in_addr Target::v4source() const {
-  const struct in_addr *addy = v4sourceip();
-  struct in_addr in;
-  if (addy) return *addy;
-  in.s_addr = 0;
-  return in;
+struct sockaddr_storage Target::source() const {
+  return sourcesock;
 }
 
 // Returns IPv4 host address or NULL if unavailable.
@@ -285,10 +372,18 @@ const struct in_addr *Target::v4sourceip() const {
   return NULL;
 }
 
+// Returns IPv6 host address or NULL if unavailable.
+const struct in6_addr *Target::v6sourceip() const {
+  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &sourcesock;
+  if (sin6->sin6_family == AF_INET6) {
+    return &(sin6->sin6_addr);
+  }
+  return NULL;
+}
 
-  /* You can set to NULL to erase a name or if it failed to resolve -- or 
+  /* You can set to NULL to erase a name or if it failed to resolve -- or
      just don't call this if it fails to resolve */
-void Target::setHostName(char *name) {
+void Target::setHostName(const char *name) {
   char *p;
   if (hostname) {
     free(hostname);
@@ -300,8 +395,8 @@ void Target::setHostName(char *name) {
       // I think only a-z A-Z 0-9 . and - are allowed, but I'll be a little more
       // generous.
       if (!isalnum((int) (unsigned char) *p) && !strchr(".-+=:_~*", *p)) {
-	log_write(LOG_STDOUT, "Illegal character(s) in hostname -- replacing with '*'\n");
-	*p = '*';
+        log_write(LOG_STDOUT, "Illegal character(s) in hostname -- replacing with '*'\n");
+        *p = '*';
       }
       p++;
     }
@@ -323,7 +418,7 @@ void Target::setTargetName(const char *name) {
      (64.71.184.53)" or "fe80::202:e3ff:fe14:1102".  The name is
      written into the buffer provided, which is also returned.  Results
      that do not fit in buflen will be truncated. */
-const char *Target::NameIP(char *buf, size_t buflen) {
+const char *Target::NameIP(char *buf, size_t buflen) const {
   assert(buf);
   assert(buflen > 8);
   if (targetname)
@@ -336,9 +431,11 @@ const char *Target::NameIP(char *buf, size_t buflen) {
 }
 
 /* This next version returns a static buffer -- so no concurrency */
-const char *Target::NameIP() {
-  if (!nameIPBuf) nameIPBuf = (char *) safe_malloc(MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
-  return NameIP(nameIPBuf, MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
+const char *Target::NameIP() const {
+  /* Add 3 characters for the hostname and IP string, hence we allocate
+    (FQDN_LEN + INET6_ADDRSTRLEN + 4) octets, with octet for the null terminator */
+  if (!nameIPBuf) nameIPBuf = (char *) safe_malloc(FQDN_LEN + INET6_ADDRSTRLEN + 4);
+  return NameIP(nameIPBuf, FQDN_LEN + INET6_ADDRSTRLEN + 4);
 }
 
   /* Returns the next hop for sending packets to this host.  Returns true if
@@ -425,7 +522,7 @@ bool Target::timedOut(const struct timeval *now) {
     used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
   }
 
-  return (used > o.host_timeout)? true : false;
+  return (used > o.host_timeout);
 }
 
 
@@ -474,14 +571,12 @@ const u8 *Target::NextHopMACAddress() const {
 }
 
 int Target::osscanPerformed(void) {
-	return osscan_flag;
+        return osscan_flag;
 }
 
 void Target::osscanSetFlag(int flag) {
-	if(osscan_flag == OS_PERF_UNREL)
-		return;
-	else
-		osscan_flag = flag;
+        if(osscan_flag == OS_PERF_UNREL)
+                return;
+        else
+                osscan_flag = flag;
 }
-
-
