@@ -6,89 +6,126 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 /* This code was originally part of the Nping tool.                        */
@@ -118,24 +155,24 @@
  *
  * 1. Imagine we need to build an ICMP echo request message that includes some
  *    arbitrary data to be echoed. We could do the following:
- * 
+ *
  *    u8 final_packet[1024];         <-- Buffer to store the resulting packet
  *    u32 final_packet_len=0;        <-- Length of the resulting packet
  *    ICMPv6Header header;           <-- The ICMPv6 fixed-length part
  *    RawData data;                  <-- The data to append to the echo message
- * 
+ *
  *    header.setType(ICMPv6_ECHO);   <-- Set ICMPv6 type to "Echo request"
  *    data.store("1234567890");      <-- Store data we need to send.
  *    header.setNextElement(&data);  <-- Tell ICMPv6Header what's after it
  *    header.setSum();               <-- Compute the checksum
- * 
+ *
  *    final_packet_len=header.dumpToBinaryBuffer(fina_packet, 1024);
  *    send_packet(final_packet, final_packet_len)
  *
  * 2. If we are sending a parameter problem message and we need to include the
  *    invoking datagram, we can call setNextElement() passing an IPv6Header
  *    pointer.
- * 
+ *
  *    u8 final_packet[1024];         <-- Buffer to store the resulting packet
  *    u32 final_packet_len=0;        <-- Length of the resulting packet
  *    ICMPv6Header header;           <-- The ICMPv6 fixed-length part
@@ -149,7 +186,7 @@
  *
  * 3. If we are sending a router solicitation message, we'll call
  *    setNextElement() passing an IPv6Options Pointer.
- * 
+ *
  *    u8 final_packet[1024];         <-- Buffer to store the resulting packet
  *    u32 final_packet_len=0;        <-- Length of the resulting packet
  *    ICMPv6Header header;           <-- The ICMPv6 fixed-length part
@@ -180,11 +217,11 @@
  * following IETF RFC documents: RFC 4443, RFC 2461, RFC 2894 */
 
 /* ICMP types and codes.
- * The following types and codes have been defined by IANA. A complete list 
+ * The following types and codes have been defined by IANA. A complete list
  * may be found at http://www.iana.org/assignments/icmpv6-parameters
  *
  * Definitions on the first level of indentation are ICMPv6 Types.
- * Definitions on the second level of indentation (values enclosed in 
+ * Definitions on the second level of indentation (values enclosed in
  * parenthesis) are ICMPv6 Codes */
 #define ICMPv6_UNREACH                      1    /* Destination unreachable  [RFC 2463, 4443] */
 #define     ICMPv6_UNREACH_NO_ROUTE        (0)   /*  --> No route to destination */
@@ -271,6 +308,14 @@
 
 
 
+/* Node Information flag bitmaks */
+#define ICMPv6_NI_FLAG_T    0x01
+#define ICMPv6_NI_FLAG_A    0x02
+#define ICMPv6_NI_FLAG_C    0x04
+#define ICMPv6_NI_FLAG_L    0x08
+#define ICMPv6_NI_FLAG_G    0x10
+#define ICMPv6_NI_FLAG_S    0x20
+
 class ICMPv6Header : public ICMPHeader {
 
         /**********************************************************************/
@@ -290,7 +335,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct nping_icmpv6_hdr nping_icmpv6_hdr_t;
 
-        
+
         /**********************************************************************/
         /* ICMPv6 MESSAGE SPECIFIC HEADERS                                    */
         /**********************************************************************/
@@ -326,7 +371,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct pkt_too_big_msg pkt_too_big_msg_t;
 
-        
+
         /* Time Exceeded Message
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |          Checksum             |
@@ -342,7 +387,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct time_exceeded_msg time_exceeded_msg_t;
 
-        
+
         /* Parameter Problem Message
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |          Checksum             |
@@ -358,7 +403,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct parameter_problem_msg parameter_problem_msg_t;
 
-        
+
         /* Echo Request/Response Messages
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |          Checksum             |
@@ -373,7 +418,7 @@ class ICMPv6Header : public ICMPHeader {
             //u8 data[?];
         }__attribute__((__packed__));
         typedef struct echo_msg echo_msg_t;
-        
+
         /* Router Advertisement Message
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |          Checksum             |
@@ -396,7 +441,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct router_advert_msg router_advert_msg_t;
 
-        
+
         /* Router Solicitation Message
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |          Checksum             |
@@ -493,7 +538,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct redirect_msg redirect_msg_t;
 
-        
+
         /* Router Renumbering Header
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |            Checksum           |
@@ -525,7 +570,7 @@ class ICMPv6Header : public ICMPHeader {
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |     Type      |     Code      |           Checksum            |
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-          |             Qtype             |             Flags             |
+          |             Qtype             |       unused      |G|S|L|C|A|T|
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           |                                                               |
           +                             Nonce                             +
@@ -538,7 +583,7 @@ class ICMPv6Header : public ICMPHeader {
         struct nodeinfo_msg{
             u16 qtype;
             u16 flags;
-            u8 nonce[NI_NONCE_LEN];
+            u64 nonce;
             //u8 data[?];
         }__attribute__((__packed__));
         typedef struct nodeinfo_msg nodeinfo_msg_t;
@@ -567,7 +612,7 @@ class ICMPv6Header : public ICMPHeader {
         }__attribute__((__packed__));
         typedef struct mld_msg mld_msg_t;
 
-        
+
         nping_icmpv6_hdr_t h;
 
         /* Helper pointers */
@@ -611,82 +656,84 @@ class ICMPv6Header : public ICMPHeader {
         int setSum();
         int setSum(u16 s);
         int setSumRandom();
-        u16 getSum();
+        u16 getSum() const;
 
         int setReserved(u32 val);
-        u32 getReserved();
+        u32 getReserved() const;
         int setUnused(u32 val);
-        u32 getUnused();
+        u32 getUnused() const;
 
         int setFlags(u8 val);
-        u8 getFlags();
+        u8 getFlags() const;
 
         int setMTU(u32 mtu);
-        u32 getMTU();
+        u32 getMTU() const;
 
         /* Parameter problem */
         int setPointer(u32 val);
-        u32 getPointer();
+        u32 getPointer() const;
 
         /* Echo */
         int setIdentifier(u16 val);
-        u16 getIdentifier();
+        u16 getIdentifier() const;
         int setSequence(u16 val);
         int setSequence(u32 val);
-        u32 getSequence();
+        u32 getSequence() const;
 
         /* Router Advertisement */
         int setCurrentHopLimit(u8 val);
-        u8 getCurrentHopLimit();
+        u8 getCurrentHopLimit() const;
 
         int setRouterLifetime(u16 val);
-        u16 getRouterLifetime();
+        u16 getRouterLifetime() const;
 
         int setReachableTime(u32 val);
-        u32 getReachableTime();
+        u32 getReachableTime() const;
 
         int setRetransmissionTimer(u32 val);
-        u32 getRetransmissionTimer();
+        u32 getRetransmissionTimer() const;
 
         int setTargetAddress(struct in6_addr addr);
-        struct in6_addr getTargetAddress();
+        struct in6_addr getTargetAddress() const;
 
         int setDestinationAddress(struct in6_addr addr);
-        struct in6_addr getDestinationAddress();
+        struct in6_addr getDestinationAddress() const;
 
         int setSegmentNumber(u8 val);
-        u8 getSegmentNumber();
+        u8 getSegmentNumber() const;
 
         int setMaxDelay(u16 val);
-        u16 getMaxDelay();
+        u16 getMaxDelay() const;
 
         /* Node Information Queries */
         int setQtype(u16 val);
-        u16 getQtype();
+        u16 getQtype() const;
         int setNodeInfoFlags(u16 val);
-        u16 getNodeInfoFlags();
+        u16 getNodeInfoFlags() const;
         int  setG(bool flag_value=true);
-        bool getG();
+        bool getG() const;
         int  setS(bool flag_value=true);
-        bool getS();
+        bool getS() const;
         int  setL(bool flag_value=true);
-        bool getL();
+        bool getL() const;
         int  setC(bool flag_value=true);
-        bool getC();
+        bool getC() const;
         int  setA(bool flag_value=true);
-        bool getA();
+        bool getA() const;
         int  setT(bool flag_value=true);
-        bool getT();
+        bool getT() const;
+        int setNonce(u64 nonce_value);
         int setNonce(const u8 *nonce);
-        u8 *getNonce();
+        u64 getNonce() const;
 
         /* Multicast Listener Discovery */
         int setMulticastAddress(struct in6_addr addr);
-        struct in6_addr getMulticastAddress();
+        struct in6_addr getMulticastAddress() const;
 
         /* Misc */
-        int getHeaderLengthFromType(u8 type);
+        int getHeaderLengthFromType(u8 type) const;
         bool isError() const;
+        const char *type2string(int type, int code) const;
 
 }; /* End of class ICMPv6Header */
 
