@@ -2,96 +2,137 @@
  * util.c -- Various utility functions.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
  * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: util.c 21905 2011-01-21 00:04:51Z fyodor $ */
+/* $Id: util.c 37126 2018-01-28 21:18:17Z fyodor $ */
 
 #include "sys_wrap.h"
 #include "util.h"
 #include "ncat.h"
+#include "nbase.h"
+#include "sockaddr_u.h"
 
-#include <assert.h>
 #include <stdio.h>
 #ifdef WIN32
 #include <iphlpapi.h>
@@ -113,22 +154,22 @@
 /* safely add 2 size_t */
 size_t sadd(size_t l, size_t r)
 {
-    size_t  t;
+    size_t t;
 
     t = l + r;
-    if(t < l)
-        bye("integer overflow %lu + %lu.", (u_long)l, (u_long)r);
+    if (t < l)
+        bye("integer overflow %lu + %lu.", (u_long) l, (u_long) r);
     return t;
 }
 
 /* safely multiply 2 size_t */
 size_t smul(size_t l, size_t r)
 {
-    size_t  t;
+    size_t t;
 
     t = l * r;
-    if(l && t / l != r)
-        bye("integer overflow %lu * %lu.", (u_long)l, (u_long)r);
+    if (l && t / l != r)
+        bye("integer overflow %lu * %lu.", (u_long) l, (u_long) r);
     return t;
 }
 
@@ -138,8 +179,8 @@ void windows_init()
     WORD werd;
     WSADATA data;
 
-    werd = MAKEWORD( 2, 2 );
-    if( (WSAStartup(werd, &data)) !=0 )
+    werd = MAKEWORD(2, 2);
+    if ((WSAStartup(werd, &data)) != 0)
         bye("Failed to start WinSock.");
 }
 #endif
@@ -154,6 +195,7 @@ void loguser(const char *fmt, ...)
     va_start(ap, fmt);
     vfprintf(stderr, fmt, ap);
     va_end(ap);
+    fflush(stderr);
 }
 
 /* Log a user message without the "Ncat: " prefix, to allow building up a line
@@ -165,6 +207,7 @@ void loguser_noprefix(const char *fmt, ...)
     va_start(ap, fmt);
     vfprintf(stderr, fmt, ap);
     va_end(ap);
+    fflush(stderr);
 }
 
 void logdebug(const char *fmt, ...)
@@ -175,12 +218,36 @@ void logdebug(const char *fmt, ...)
     va_start(ap, fmt);
     vfprintf(stderr, fmt, ap);
     va_end(ap);
+    fflush(stderr);
+}
+
+void logtest(const char *fmt, ...)
+{
+    va_list ap;
+
+    fprintf(stderr, "NCAT TEST: ");
+    va_start(ap, fmt);
+    vfprintf(stderr, fmt, ap);
+    va_end(ap);
+    fflush(stderr);
 }
 
 /* Exit status 2 indicates a program error other than a network error. */
 void die(char *err)
 {
+#ifdef WIN32
+  int error_number;
+  char *strerror_s;
+  error_number = GetLastError();
+  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
+      NULL, error_number, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+      (LPTSTR) &strerror_s,  0, NULL);
+  fprintf(stderr, "%s: %s\n", err, strerror_s);
+  HeapFree(GetProcessHeap(), 0, strerror_s);
+#else
     perror(err);
+#endif
+    fflush(stderr);
     exit(2);
 }
 
@@ -194,6 +261,7 @@ void bye(const char *fmt, ...)
     vfprintf(stderr, fmt, ap);
     va_end(ap);
     fprintf(stderr, " QUITTING.\n");
+    fflush(stderr);
 
     exit(2);
 }
@@ -208,11 +276,11 @@ void zmem(void *mem, size_t n)
    buffer and updates the variables to make room if necessary. */
 int strbuf_append(char **buf, size_t *size, size_t *offset, const char *s, size_t n)
 {
-    assert(*offset <= *size);
+    ncat_assert(*offset <= *size);
 
     if (n >= *size - *offset) {
         *size += n + 1;
-        *buf = (char*) safe_realloc(*buf, *size);
+        *buf = (char *) safe_realloc(*buf, *size);
     }
 
     memcpy(*buf + *offset, s, n);
@@ -235,11 +303,11 @@ int strbuf_sprintf(char **buf, size_t *size, size_t *offset, const char *fmt, ..
     va_list va;
     int n;
 
-    assert(*offset <= *size);
+    ncat_assert(*offset <= *size);
 
     if (*buf == NULL) {
         *size = 1;
-        *buf = (char*) safe_malloc(*size);
+        *buf = (char *) safe_malloc(*size);
     }
 
     for (;;) {
@@ -252,26 +320,13 @@ int strbuf_sprintf(char **buf, size_t *size, size_t *offset, const char *fmt, ..
             *size += n + 1;
         else
             break;
-        *buf = (char*) safe_realloc(*buf, *size);
+        *buf = (char *) safe_realloc(*buf, *size);
     }
     *offset += n;
 
     return n;
 }
 
-/* Make a new allocated null-terminated string from the bytes [start, end). */
-char *mkstr(const char *start, const char *end)
-{
-    char *s;
-
-    assert(end >= start);
-    s = (char *) safe_malloc(end - start + 1);
-    memcpy(s, start, end - start);
-    s[end - start] = '\0';
-
-    return s;
-}
-
 /* Return true if the given address is a local one. */
 int addr_is_local(const union sockaddr_u *su)
 {
@@ -305,8 +360,8 @@ int addr_is_local(const union sockaddr_u *su)
         if (addr->ai_family != su->storage.ss_family)
             continue;
         if (addr->ai_addrlen > sizeof(addr_su)) {
-            bye("getaddrinfo returned oversized address (%u > %u)",
-                addr->ai_addrlen, sizeof(addr_su));
+            bye("getaddrinfo returned oversized address (%lu > %lu)",
+                (unsigned long) addr->ai_addrlen, (unsigned long) sizeof(addr_su));
         }
         memcpy(&addr_su, addr->ai_addr, addr->ai_addrlen);
         if (su->storage.ss_family == AF_INET) {
@@ -317,18 +372,20 @@ int addr_is_local(const union sockaddr_u *su)
                 break;
         }
     }
-    freeaddrinfo(addrs);
-    if (addr != NULL)
+    if (addr != NULL) {
+        freeaddrinfo(addrs);
         return 1;
-
-    return 0;
+    } else {
+        return 0;
+    }
 }
 
 /* Converts an IP address given in a sockaddr_u to an IPv4 or
    IPv6 IP address string.  Since a static buffer is returned, this is
    not thread-safe and can only be used once in calls like printf()
 */
-const char *inet_socktop(const union sockaddr_u *su) {
+const char *inet_socktop(const union sockaddr_u *su)
+{
     static char buf[INET6_ADDRSTRLEN + 1];
     void *addr;
 
@@ -363,60 +420,130 @@ unsigned short inet_port(const union sockaddr_u *su)
     return 0;
 }
 
-int do_listen(int type, int proto)
+/* Return a listening socket after setting various characteristics on it.
+   Returns -1 on error. */
+int do_listen(int type, int proto, const union sockaddr_u *srcaddr_u)
 {
     int sock = 0, option_on = 1;
+    size_t sa_len;
 
-    if(type != SOCK_STREAM && type != SOCK_DGRAM)
+    if (type != SOCK_STREAM && type != SOCK_DGRAM)
         return -1;
 
     /* We need a socket that can be inherited by child processes in
        ncat_exec_win.c, for --exec and --sh-exec. inheritable_socket is from
        nbase. */
-    sock = inheritable_socket(srcaddr.storage.ss_family, type, proto);
+    sock = inheritable_socket(srcaddr_u->storage.ss_family, type, proto);
+    if (sock < 0)
+        return -1;
 
     Setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &option_on, sizeof(int));
 
-    if (bind(sock, &srcaddr.sockaddr, (int) srcaddrlen) < 0) {
-        bye("bind to %s:%hu: %s.", inet_socktop(&srcaddr),
-            inet_port(&srcaddr), socket_strerror(socket_errno()));
+/* IPPROTO_IPV6 is defined in Visual C++ only when _WIN32_WINNT >= 0x501.
+   Nbase's nbase_winunix.h defines _WIN32_WINNT to a lower value for
+   compatibility with older versions of Windows. This code disables IPv6 sockets
+   that also receive IPv4 connections. This is the default on Windows anyway so
+   it doesn't make a difference.
+   http://support.microsoft.com/kb/950688
+   http://msdn.microsoft.com/en-us/library/bb513665
+*/
+#ifdef IPPROTO_IPV6
+#ifdef IPV6_V6ONLY
+    if (srcaddr_u->storage.ss_family == AF_INET6) {
+        int set = 1;
+        /* Tell it to not try and bind to IPV4 */
+        if (setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, &set, sizeof(set)) == -1)
+            die("Unable to set IPV6 socket to bind only to IPV6");
+    }
+#endif
+#endif
+
+    switch(srcaddr_u->storage.ss_family) {
+#ifdef HAVE_SYS_UN_H
+      case AF_UNIX:
+        sa_len = SUN_LEN(&srcaddr_u->un);
+        break;
+#endif
+#ifdef HAVE_SOCKADDR_SA_LEN
+      default:
+        sa_len = srcaddr_u->sockaddr.sa_len;
+        break;
+#else
+      case AF_INET:
+        sa_len = sizeof (struct sockaddr_in);
+        break;
+#ifdef AF_INET6
+      case AF_INET6:
+        sa_len = sizeof (struct sockaddr_in6);
+        break;
+#endif
+      default:
+        sa_len = sizeof(*srcaddr_u);
+        break;
+#endif
     }
 
-    if(type == SOCK_STREAM)
+    if (bind(sock, &srcaddr_u->sockaddr, sa_len) < 0) {
+#ifdef HAVE_SYS_UN_H
+        if (srcaddr_u->storage.ss_family == AF_UNIX)
+            bye("bind to %s: %s.", srcaddr_u->un.sun_path,
+                socket_strerror(socket_errno()));
+        else
+#endif
+            bye("bind to %s:%hu: %s.", inet_socktop(srcaddr_u),
+                inet_port(srcaddr_u), socket_strerror(socket_errno()));
+    }
+
+    if (type == SOCK_STREAM)
         Listen(sock, BACKLOG);
 
-    if(o.verbose)
-        loguser("Listening on %s:%hu\n", inet_socktop(&srcaddr), inet_port(&srcaddr));
+    if (o.verbose) {
+#ifdef HAVE_SYS_UN_H
+        if (srcaddr_u->storage.ss_family == AF_UNIX)
+            loguser("Listening on %s\n", srcaddr_u->un.sun_path);
+        else
+#endif
+            loguser("Listening on %s:%hu\n", inet_socktop(srcaddr_u), inet_port(srcaddr_u));
+    }
+    if (o.test)
+        logtest("LISTEN\n");
 
     return sock;
 }
 
 int do_connect(int type)
 {
-    int sock=0;
+    int sock = 0;
 
-    if(type != SOCK_STREAM && type != SOCK_DGRAM)
+    if (type != SOCK_STREAM && type != SOCK_DGRAM)
         return -1;
 
     /* We need a socket that can be inherited by child processes in
        ncat_exec_win.c, for --exec and --sh-exec. inheritable_socket is from
        nbase. */
-    sock = inheritable_socket(targetss.storage.ss_family, type, 0);
+    sock = inheritable_socket(targetaddrs->addr.storage.ss_family, type, 0);
 
     if (srcaddr.storage.ss_family != AF_UNSPEC) {
-        if (bind(sock, &srcaddr.sockaddr, (int) srcaddrlen) < 0) {
+        size_t sa_len;
+
+#ifdef HAVE_SOCKADDR_SA_LEN
+        sa_len = srcaddr.sockaddr.sa_len;
+#else
+        sa_len = sizeof(srcaddr);
+#endif
+        if (bind(sock, &srcaddr.sockaddr, sa_len) < 0) {
             bye("bind to %s:%hu: %s.", inet_socktop(&srcaddr),
                 inet_port(&srcaddr), socket_strerror(socket_errno()));
         }
     }
 
-    if(sock != -1){
-       if(connect(sock, &targetss.sockaddr, (int) targetsslen)!= -1)
-          return sock;
-       else if(socket_errno()==EINPROGRESS||socket_errno()==EAGAIN)
-          return sock;
+    if (sock != -1) {
+        if (connect(sock, &targetaddrs->addr.sockaddr, (int) targetaddrs->addrlen) != -1)
+            return sock;
+        else if (socket_errno() == EINPROGRESS || socket_errno() == EAGAIN)
+            return sock;
     }
-    return -1 ;
+    return -1;
 }
 
 unsigned char *buildsrcrte(struct in_addr dstaddr, struct in_addr routes[],
@@ -462,6 +589,17 @@ int allow_access(const union sockaddr_u *su)
     return 1;
 }
 
+/*
+ * Fills the given timeval struct with proper
+ * values based on the given time in milliseconds.
+ * The pointer to timeval struct must NOT be NULL.
+ */
+void ms_to_timeval(struct timeval *tv, long ms)
+{
+    tv->tv_sec = ms / 1000;
+    tv->tv_usec = (ms - (tv->tv_sec * 1000)) * 1000;
+}
+
 /*
  * ugly code to maintain our list of fds so we can have proper fdmax for
  * select().  really this should be generic list code, not this silly bit of
@@ -471,17 +609,17 @@ int allow_access(const union sockaddr_u *su)
 /* add an fdinfo to our list */
 int add_fdinfo(fd_list_t *fdl, struct fdinfo *s)
 {
-    if(fdl->nfds >= fdl->maxfds)
+    if (fdl->nfds >= fdl->maxfds)
         return -1;
 
     fdl->fds[fdl->nfds] = *s;
 
     fdl->nfds++;
 
-    if(s->fd > fdl->fdmax)
+    if (s->fd > fdl->fdmax)
         fdl->fdmax = s->fd;
 
-    if(o.debug > 1)
+    if (o.debug > 1)
         logdebug("Added fd %d to list, nfds %d, maxfd %d\n", s->fd, fdl->nfds, fdl->fdmax);
     return 0;
 }
@@ -504,20 +642,20 @@ int rm_fd(fd_list_t *fdl, int fd)
     int x = 0, last = fdl->nfds;
 
     /* make sure we have a list */
-    if(last == 0)
+    if (last == 0)
         bye("Program bug: Trying to remove fd from list with no fds.");
 
     /* find the fd in the list */
-    for(x = 0; x < last; x++)
-        if(fdl->fds[x].fd == fd)
+    for (x = 0; x < last; x++)
+        if (fdl->fds[x].fd == fd)
             break;
 
     /* make sure we found it */
-    if(x == last)
+    if (x == last)
         bye("Program bug: fd (%d) not on list.", fd);
 
-    /* remove it, does nothing if(last == 1) */
-    if(o.debug > 1)
+    /* remove it, does nothing if (last == 1) */
+    if (o.debug > 1)
         logdebug("Swapping fd[%d] (%d) with fd[%d] (%d)\n",
                  x, fdl->fds[x].fd, last - 1, fdl->fds[last - 1].fd);
     fdl->fds[x] = fdl->fds[last - 1];
@@ -525,10 +663,10 @@ int rm_fd(fd_list_t *fdl, int fd)
     fdl->nfds--;
 
     /* was it the max */
-    if(fd == fdl->fdmax)
+    if (fd == fdl->fdmax)
         fdl->fdmax = get_maxfd(fdl);
 
-    if(o.debug > 1)
+    if (o.debug > 1)
         logdebug("Removed fd %d from list, nfds %d, maxfd %d\n", fd, fdl->nfds, fdl->fdmax);
     return 0;
 }
@@ -536,10 +674,10 @@ int rm_fd(fd_list_t *fdl, int fd)
 /* find the max descriptor in our list */
 int get_maxfd(fd_list_t *fdl)
 {
-    int x = 0,  max = -1,   nfds = fdl->nfds;
+    int x = 0, max = -1, nfds = fdl->nfds;
 
-    for(x = 0; x < nfds; x++)
-        if(fdl->fds[x].fd > max)
+    for (x = 0; x < nfds; x++)
+        if (fdl->fds[x].fd > max)
             max = fdl->fds[x].fd;
 
     return max;
@@ -563,7 +701,7 @@ void init_fdlist(fd_list_t *fdl, int maxfds)
     fdl->fdmax = -1;
     fdl->maxfds = maxfds;
 
-    if(o.debug > 1)
+    if (o.debug > 1)
         logdebug("Initialized fdlist with %d maxfds\n", maxfds);
 }
 
@@ -589,7 +727,7 @@ void free_fdlist(fd_list_t *fdl)
 int fix_line_endings(char *src, int *len, char **dst, int *state)
 {
     int fix_count;
-    int i,j;
+    int i, j;
     int num_bytes = *len;
     int prev_state = *state;
 
@@ -600,21 +738,22 @@ int fix_line_endings(char *src, int *len, char **dst, int *state)
     /* get count of \n without matching \r */
     fix_count = 0;
     for (i = 0; i < num_bytes; i++) {
-        if (src[i] == '\n' && ((i == 0) ? !prev_state : src[i-1] != '\r'))
+        if (src[i] == '\n' && ((i == 0) ? !prev_state : src[i - 1] != '\r'))
             fix_count++;
     }
-    if (fix_count <= 0 ) return 0;
+    if (fix_count <= 0)
+        return 0;
 
     /* now insert matching \r */
     *dst = (char *) safe_malloc(num_bytes + fix_count);
-    j=0;
+    j = 0;
 
     for (i = 0; i < num_bytes; i++) {
-        if (src[i] == '\n' && ((i == 0) ? !prev_state : src[i-1] != '\r')) {
-            memcpy(*dst+j, "\r\n", 2);
+        if (src[i] == '\n' && ((i == 0) ? !prev_state : src[i - 1] != '\r')) {
+            memcpy(*dst + j, "\r\n", 2);
             j += 2;
         } else {
-            memcpy(*dst+j, src+i, 1);
+            memcpy(*dst + j, src + i, 1);
             j++;
         }
     }
@@ -622,3 +761,39 @@ int fix_line_endings(char *src, int *len, char **dst, int *state)
 
     return 1;
 }
+
+/*-
+ * next_protos_parse parses a comma separated list of strings into a string
+ * in a format suitable for passing to SSL_CTX_set_next_protos_advertised.
+ *   outlen: (output) set to the length of the resulting buffer on success.
+ *   err: NULL on failure
+ *   in: a NULL terminated string like "abc,def,ghi"
+ *
+ *   returns: a malloc'd buffer or NULL on failure.
+ */
+unsigned char *next_protos_parse(size_t *outlen, const char *in)
+{
+    size_t len;
+    unsigned char *out;
+    size_t i, start = 0;
+
+    len = strlen(in);
+    if (len >= 65535)
+        return NULL;
+
+    out = (unsigned char *)safe_malloc(strlen(in) + 1);
+    for (i = 0; i <= len; ++i) {
+        if (i == len || in[i] == ',') {
+            if (i - start > 255) {
+                free(out);
+                return NULL;
+            }
+            out[start] = i - start;
+            start = i + 1;
+        } else
+            out[i + 1] = in[i];
+    }
+
+    *outlen = len + 1;
+    return out;
+}
