@@ -7,89 +7,126 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
@@ -98,6 +135,10 @@
 #include "nping_winconfig.h"
 #endif
 
+#ifndef FQDN_LEN
+#define FQDN_LEN 254
+#endif
+
 #include "NpingTarget.h"
 #include <dnet.h>
 #include "nbase.h"
@@ -208,7 +249,7 @@ int NpingTarget::getTargetSockAddr(struct sockaddr_storage *ss, size_t *ss_len)
   memcpy(ss, &targetsock, targetsocklen);
   *ss_len = targetsocklen;
   return 0;
-  
+
 } /* End of getTargetSockAddr() */
 
 
@@ -545,7 +586,7 @@ void NpingTarget::setResolvedHostName(char *name) {
       // I think only a-z A-Z 0-9 . and - are allowed, but I'll be a little more
       // generous.
       if (!isalnum(*p) && !strchr(".-+=:_~*", *p)) {
-    outError(QT_2, "Illegal character(s) in hostname -- replacing with '*'\n");
+    nping_warning(QT_2, "Illegal character(s) in hostname -- replacing with '*'\n");
     *p = '*';
       }
       p++;
@@ -592,7 +633,7 @@ int NpingTarget::setNamedHost(bool val){
 
 bool NpingTarget::isNamedHost(){
   assert(this->namedhost==1 || this->namedhost==0 );
-  return (this->namedhost==1) ? true : false;
+  return (this->namedhost==1);
 } /* End of isNamedHost() */
 
 
@@ -607,10 +648,10 @@ void NpingTarget::generateIPString() {
   }else if(sin->sin_family == AF_INET6){
     ret=inet_ntop(AF_INET6, (char *) &sin6->sin6_addr, targetipstring, sizeof(targetipstring));
   }else{
-    outFatal(QT_3, "NpingTarget::GenerateIPString(): Unsupported address family");
+    nping_fatal(QT_3, "NpingTarget::GenerateIPString(): Unsupported address family");
   }
   if( ret==NULL )
-    outFatal(QT_3, "NpingTarget::GenerateIPString(): Unsupported address family");
+    nping_fatal(QT_3, "NpingTarget::GenerateIPString(): Unsupported address family");
  targetipstring_set=true;
 } /* End of generateIPString() */
 
@@ -628,7 +669,7 @@ const char *NpingTarget::getSourceIPStr() {
   }else if(sin->sin_family == AF_INET6){
     ret=inet_ntop(AF_INET6, (char *) &sin6->sin6_addr, buffer, sizeof(buffer));
   }else{
-    outFatal(QT_3, "NpingTarget::getSourceIPString(): Unsupported address family");
+    nping_fatal(QT_3, "NpingTarget::getSourceIPString(): Unsupported address family");
   }
   if(ret==NULL)
     return NULL;
@@ -652,7 +693,7 @@ const char *NpingTarget::getSpoofedSourceIPStr() {
   }else if(sin->sin_family == AF_INET6){
     ret=inet_ntop(AF_INET6, (char *) &sin6->sin6_addr, buffer, sizeof(buffer));
   }else{
-    outFatal(QT_3, "NpingTarget::getSourceIPString(): Unsupported address family");
+    nping_fatal(QT_3, "NpingTarget::getSourceIPString(): Unsupported address family");
   }
   if(ret==NULL)
     return NULL;
@@ -671,7 +712,7 @@ const char *NpingTarget::getNextHopIPStr(){
   }else if(sin->sin_family == AF_INET6){
     ret=inet_ntop(AF_INET6, (char *) &sin6->sin6_addr, buffer, sizeof(buffer));
   }else{
-    outFatal(QT_3, "NpingTarget::getNextHopIPStr(): Unsupported address family");
+    nping_fatal(QT_3, "NpingTarget::getNextHopIPStr(): Unsupported address family");
   }
   if(ret==NULL)
     return NULL;
@@ -731,8 +772,8 @@ const char *NpingTarget::getNameAndIP(char *buf, size_t buflen) {
 /** This next version returns a static buffer -- so no concurrency */
 const char *NpingTarget::getNameAndIP() {
   if(!nameIPBuf)
-    nameIPBuf = (char *)safe_malloc(MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
-  return getNameAndIP(nameIPBuf, MAXHOSTNAMELEN + INET6_ADDRSTRLEN);
+    nameIPBuf = (char *)safe_malloc(FQDN_LEN + INET6_ADDRSTRLEN + 4);
+  return getNameAndIP(nameIPBuf, FQDN_LEN + INET6_ADDRSTRLEN + 4);
 } /* End of getNameAndIP() */
 
 
@@ -774,7 +815,7 @@ bool NpingTarget::determineNextHopMACAddress() {
   if ( this->getNextHopMACAddress() )
     return true;
 
-  outPrint(DBG_2,"Determining target %s MAC address or next hop MAC address...", this->getTargetIPstr() );
+  nping_print(DBG_2,"Determining target %s MAC address or next hop MAC address...", this->getTargetIPstr() );
   /* For connected machines, it is the same as the target addy */
   if (this->isDirectlyConnected() && this->getMACAddress() ) {
     this->setNextHopMACAddress(this->getMACAddress());
@@ -795,45 +836,45 @@ bool NpingTarget::determineNextHopMACAddress() {
   }
 
   /* Maybe the system ARP cache will be more helpful */
-  outPrint(DBG_3,"    > Checking system's ARP cache...");
+  nping_print(DBG_3,"    > Checking system's ARP cache...");
   a = arp_open();
   addr_ston((sockaddr *)&targetss, &ae.arp_pa);
   if (arp_get(a, &ae) == 0) {
     mac_cache_set(&targetss, ae.arp_ha.addr_eth.data);
     this->setNextHopMACAddress(ae.arp_ha.addr_eth.data);
     arp_close(a);
-    outPrint(DBG_3,"    > Success: Entry found [%s]", this->getNextHopMACStr() );
+    nping_print(DBG_3,"    > Success: Entry found [%s]", this->getNextHopMACStr() );
     return true;
   }
   arp_close(a);
-  outPrint(DBG_3,"    > No relevant entries found in system's ARP cache.");
+  nping_print(DBG_3,"    > No relevant entries found in system's ARP cache.");
 
 
   /* OK, the last choice is to send our own damn ARP request (and
      retransmissions if necessary) to determine the MAC */
   /* We first try sending the ARP with our spoofed IP address on it */
   if( this->spoofingSourceAddress() ){
-    outPrint(DBG_3,"    > Sending ARP request using spoofed IP %s...", this->getSpoofedSourceIPStr() );
+    nping_print(DBG_3,"    > Sending ARP request using spoofed IP %s...", this->getSpoofedSourceIPStr() );
       this->getSpoofedSourceSockAddr(&srcss, NULL);
       if (doArp(this->getDeviceName(), this->getSrcMACAddress(), &srcss, &targetss, mac, NULL)) {
         mac_cache_set(&targetss, mac);
         this->setNextHopMACAddress(mac);
-        outPrint(DBG_4,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
+        nping_print(DBG_4,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
         return true;
       }
   }
-  outPrint(DBG_3,"    > No ARP responses received." );
+  nping_print(DBG_3,"    > No ARP responses received." );
 
   /* If our spoofed IP address didn't work, try our real IP */
-  outPrint(DBG_4,"    > Sending ARP request using our real IP %s...", this->getSourceIPStr() );
+  nping_print(DBG_4,"    > Sending ARP request using our real IP %s...", this->getSourceIPStr() );
   this->getSourceSockAddr(&srcss, NULL);
   if (doArp(this->getDeviceName(), this->getSrcMACAddress(), &srcss, &targetss, mac, NULL)) {
     mac_cache_set(&targetss, mac);
     this->setNextHopMACAddress(mac);
-    outPrint(DBG_3,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
+    nping_print(DBG_3,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
     return true;
   }
-  outPrint(DBG_3,"    > No ARP responses received" );
+  nping_print(DBG_3,"    > No ARP responses received" );
 
   /* I'm afraid that we couldn't find it!  Maybe it doesn't exist?*/
   return false;
@@ -921,7 +962,7 @@ int NpingTarget::setProbeRecvTCP(u16 sport, u16 dport){
           /* Update stats info */
           diff= TIMEVAL_SUBTRACT(this->sentprobes[i].recv, this->sentprobes[i].sent);
           this->updateRTTs(diff);
-  
+
         return OP_SUCCESS;
     }
   }
@@ -964,7 +1005,7 @@ int NpingTarget::setProbeSentICMP(u16 id, u16 seq){
   current_stat=(current_stat+1)%MAX_SENTPROBEINFO_ENTRIES;
   if( total_stats< MAX_SENTPROBEINFO_ENTRIES)
     total_stats++;
-    return OP_SUCCESS;
+  return OP_SUCCESS;
 } /* End of setProbeSentARP() */
 
 
@@ -1026,10 +1067,10 @@ int NpingTarget::updateRTTs(unsigned long int diff){
 
 
 int NpingTarget::printStats(){
-  outPrint(VB_0, "Statistics for host %s:", this->getNameAndIP());
-  outPrint(VB_0|NO_NEWLINE," |  ");
+  nping_print(VB_0, "Statistics for host %s:", this->getNameAndIP());
+  nping_print(VB_0|NO_NEWLINE," |  ");
   this->printCounts();
-  outPrint(VB_0|NO_NEWLINE," |_ ");
+  nping_print(VB_0|NO_NEWLINE," |_ ");
   this->printRTTs();
   return OP_SUCCESS;
 } /* End of printStats() */
@@ -1040,11 +1081,11 @@ void NpingTarget::printCounts(){
   unsigned long int lost = this->sent_total - this->recv_total;
 
   /* Sent Packets */
-  outPrint(VB_0|NO_NEWLINE, "Probes Sent: %ld ", this->sent_total);
+  nping_print(VB_0|NO_NEWLINE, "Probes Sent: %ld ", this->sent_total);
   /* Received Packets */
-  outPrint(VB_0|NO_NEWLINE,"| Rcvd: %ld ", this->recv_total );
+  nping_print(VB_0|NO_NEWLINE,"| Rcvd: %ld ", this->recv_total );
   /* Lost Packets */
-  outPrint(VB_0|NO_NEWLINE,"| Lost: %ld ", lost );
+  nping_print(VB_0|NO_NEWLINE,"| Lost: %ld ", lost );
 
   /* Only compute percentage if we actually sent packets, don't do divisions
    * by zero! (this could happen when user presses CTRL-C and we print the
@@ -1052,24 +1093,24 @@ void NpingTarget::printCounts(){
   float percentlost=0.0;
   if( lost!=0 && this->sent_total!=0)
     percentlost=((double)lost)/((double)this->sent_total) * 100;    
-  outPrint(VB_0," (%.2lf%%)", percentlost);
+  nping_print(VB_0," (%.2lf%%)", percentlost);
 } /* End of printCounts() */
 
 
 /* Print round trip times */
 void NpingTarget::printRTTs(){
   if( max_rtt_set )
-    outPrint(VB_0|NO_NEWLINE,"Max rtt: %.3lfms ", this->max_rtt/1000.0 );
+    nping_print(QT_1|NO_NEWLINE,"Max rtt: %.3lfms ", this->max_rtt/1000.0 );
   else
-    outPrint(VB_0|NO_NEWLINE,"Max rtt: N/A ");
+    nping_print(QT_1|NO_NEWLINE,"Max rtt: N/A ");
 
   if( min_rtt_set )  
-    outPrint(VB_0|NO_NEWLINE,"| Min rtt: %.3lfms ", this->min_rtt/1000.0 );
+    nping_print(QT_1|NO_NEWLINE,"| Min rtt: %.3lfms ", this->min_rtt/1000.0 );
   else
-    outPrint(VB_0|NO_NEWLINE,"| Min rtt: N/A " );
+    nping_print(QT_1|NO_NEWLINE,"| Min rtt: N/A " );
 
   if( avg_rtt_set)  
-    outPrint(VB_0,"| Avg rtt: %.3lfms", this->avg_rtt/1000.0 );
+    nping_print(QT_1,"| Avg rtt: %.3lfms", this->avg_rtt/1000.0 );
   else
-    outPrint(VB_0,"| Avg rtt: N/A" );
+    nping_print(QT_1,"| Avg rtt: N/A" );
 } /* End of printRTTs() */
