@@ -4,89 +4,126 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
@@ -154,10 +191,6 @@
     #include <sys/param.h> /* Defines MAXHOSTNAMELEN on BSD*/
 #endif
 
-#if HAVE_RPC_TYPES_H
-    #include <rpc/types.h>
-#endif
-
 #if HAVE_SYS_SOCKET_H
     #include <sys/socket.h>
 #endif
@@ -204,7 +237,7 @@
 
 /* CONSTANT DEFINES ***********************************************************
  * @warning It's better not to play with these, because the code may make     *
- * SOME asumptions like "defined value A is an integer greater than defined   *
+ * SOME assumptions like "defined value A is an integer greater than defined  *
  * value B" or "value C is an odd integer greater than 0", etc.               */
 
 /* VERBOSITY LEVELS */
@@ -215,18 +248,18 @@
  * printed. For example, we have some calls to output functions that print out
  * things like "%d target IPs resolved". We don't want that message to always
  * get printed during  Nping's execution. We only want it out when the user
- * has increasse the verbosity.
+ * has increase the verbosity.
  * 
  * So the thing here is that there are two things that should be taken
  * into account:
  *  1. The current verbosity level that user has supplied from the command line
- *  2. The verbosity level that we supply in our print calls ( outPrint(),
- *     outError(), etc...)
+ *  2. The verbosity level that we supply in our print calls ( nping_print(),
+ *     nping_warning(), etc...)
  *
  *  Fortunately Nping output functions already take care of checking the
  *  current verbosity level, so programmers only have to decide which level
  *  should they specify in their output calls. If you are a programmer and
- *  you are using outPrint(), outError() or outFatal() calls in Nping's code,
+ *  you are using nping_print(), nping_warning() or nping_fatal() calls in Nping's code,
  *  you have to ask yourself: Do I want to print extra information that
  *  shouldn't be printed by default? Or am I printing important stuff like
  *  errors, etc, that should almost always be printed out?
@@ -244,16 +277,16 @@
  *  verbose to us, programmers, because calls that supply QT_X levels almost
  *  always get printed. This is because base verbosity is VB_0 and that
  *  includes all QT_X levels. So you have to be careful with those. QT_ levels
- *  should only be used to print important stuff like fata errors, warnings,
+ *  should only be used to print important stuff like fatal errors, warnings,
  *  and some basic running time information. Level QT_4 is the quiet-est one
  *  and nothing is ever printed out.
  *
  *  Check the comments after each level definition to see how they should be
  *  used. Here are some examples:
  * 
- *  outFatal(QT_3,"createIPv4(): NULL pointer supplied.");
- *  outPrint(DBG_2,"Resolving specified targets...");
- *  outPrint(VB_0, "Raw packets sent: %llu ", this->stats.getSentPackets() );
+ *  nping_fatal(QT_3,"createIPv4(): NULL pointer supplied.");
+ *  nping_print(DBG_2,"Resolving specified targets...");
+ *  nping_print(VB_0, "Raw packets sent: %llu ", this->stats.getSentPackets() );
  *
  * */
 
@@ -264,8 +297,8 @@
 #define QT_1 3   /**< Start and timing information but no sent/recv packets  */
 
 /* Base level (QT_0 is provided for consistency but should not be used)      */
-#define QT_0 4   /**< Normal info (sent/recv packes, statistics...) (DEFAULT */
-#define VB_0 4   /**< Normal info (sent/recv packes, statistics...) (DEFAULT)*/
+#define QT_0 4   /**< Normal info (sent/recv packets, statistics...) (DEFAULT */
+#define VB_0 4   /**< Normal info (sent/recv packets, statistics...) (DEFAULT)*/
 
 /* More verbosity */
 #define VB_1 5   /**< Detailed information about times, flags, etc.          */
@@ -293,7 +326,7 @@
 
 #define MAX_DEV_LEN 128           /**< Max network interface name length     */
 
-#define NO_NEWLINE 0x8000 /**< Used in outFatal(), outError() and outPrint() */
+#define NO_NEWLINE 0x8000 /**< Used in nping_fatal(), nping_warning() and nping_print() */
 
 /** Bit count for number parsing functions */
 #define RANGE_8_BITS  8
@@ -308,8 +341,8 @@
 
 /* General tunable defines  **************************************************/
 #define NPING_NAME "Nping"
-#define NPING_URL "http://nmap.org/nping"
-#define NPING_VERSION "0.6.01"
+#define NPING_URL "https://nmap.org/nping"
+#define NPING_VERSION "0.7.70"
 
 
 #define DEFAULT_VERBOSITY VB_0
@@ -322,18 +355,18 @@
 /* When doing traceroute, the number of packets sent to each host must be
  * higher because 5 is probably not enough to reach the average target on the
  * Internet. The following paper suggests that internet hosts are no more than
- * 30 hops apart, so setting the packet count to 32 when --traceroute is set
+ * 30 hops apart, so setting the packet count to 48 when --traceroute is set
  * seems like a safe choice.
  *    Cheng, J., Haining, W. and Kang, GS. (2006). Hop-Count Filtering: An
  *    Effective Defense Against Spoofed DDoS Traffic. Australian Telecommu-
  *    nication Networks & Applications Conference (ATNAC). Australia.
  *    <http://portal.acm.org/citation.cfm?id=948109.948116>
  */
-#define TRACEROUTE_PACKET_COUNT 32
+#define TRACEROUTE_PACKET_COUNT 48
 
 #define DEFAULT_DELAY 1000              /**< Milliseconds between each probe */
 
- /** Millisenconds Nping waits for teplies after all probes have been  sent */
+ /** Milliseconds Nping waits for replies after all probes have been sent */
 #define DEFAULT_WAIT_AFTER_PROBES 1000 
 
 #define DEFAULT_IP_TTL 64               /**< Default IP Time To Live         */
@@ -368,7 +401,7 @@
  * of the whole IP packet is more than 16436 when using loopback interface or
  * more than 1500 when using a normal network interface, the kernel complains
  * and says "Message too long". This is obviously caused by the configured
- * MTU. So the thing is that altough we allow users to specify payloads up to
+ * MTU. So the thing is that although we allow users to specify payloads up to
  * MAX_PAYLOAD_ALLOWED bytes, when we generate random payloads, we set our
  * on limit on 1500-20-20=1460 bytes. Let's be conservative and consider that
  * IP packet has 40bytes of options and TCP has 20. So max length should be
@@ -394,14 +427,14 @@
  * without problem, so it should never happen. We expect to use this constant
  * when received packets are really weird (eg. tunneled traffic, protocols we
  * don't understand, etc. The 40 bytes are a compromise between dropping the
- * packet but provide total protection against data leakeage due to attacks to
+ * packet but provide total protection against data leakage due to attacks to
  * the echo server, and providing some flexibility at the risk of leaking
  * a few bytes if an attacker is able to trick the echo server into echoing
  * packets that were not originated by him. */
 #define PAYLOAD_ECHO_BYTES_IN_DOUBT 40
 
 #define NSOCK_INFINITE -1
-    
+
 /* Prototypes for nping.cc shared functions */
 char *getBPFFilterString();
 
