@@ -1,22 +1,22 @@
 
 /***************************************************************************
- * utils_net.cc -- Miscellanious network-related functions that perform    *
+ * utils_net.cc -- Miscellaneous network-related functions that perform    *
  * various tasks.                                                          *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
- * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we sell    *
- * alternative licenses (contact sales@insecure.com).  Dozens of software  *
- * vendors already license Nmap technology such as host discovery, port    *
- * scanning, OS detection, version detection, and the Nmap Scripting       *
- * Engine.                                                                 *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
  * Note that the GPL places important restrictions on "derivative works",  *
  * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -58,18 +58,25 @@
  * particularly including the GPL Section 3 requirements of providing      *
  * source code and allowing free redistribution of the work as a whole.    *
  *                                                                         *
- * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
  * linked combinations including the two.                                  *
  *                                                                         *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
  * Any redistribution of Covered Software, including any derived works,    *
  * must obey and carry forward all of the terms of this license, including *
  * obeying all GPL rules and restrictions.  For example, source code of    *
  * the whole work must be provided and free redistribution must be         *
  * allowed.  All GPL references to "this License", are to be treated as    *
- * including the special and conditions of the license text as well.       *
+ * including the terms and conditions of this license text as well.        *
  *                                                                         *
  * Because this license imposes special exceptions to the GPL, Covered     *
  * Work may not be combined (even as part of a larger work) with plain GPL *
@@ -82,34 +89,33 @@
  * Covered Software without special permission from the copyright holders. *
  *                                                                         *
  * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to the dev@nmap.org mailing list for possible incorporation into the    *
  * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
@@ -118,7 +124,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
  * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 #include "nping.h"
@@ -471,7 +477,7 @@ int getPacketStrInfo(const char *proto, const u8 *packet, u32 len, u8 *dstbuff,
     else
         nping_fatal(QT_3, "getPacketStrInfo(): Unable to determinate transport layer protocol");
   }else{
-    nping_fatal(QT_3, "getPacketStrInfo(): Unkwnown protocol");
+    nping_fatal(QT_3, "getPacketStrInfo(): Unknown protocol");
   }
   return OP_SUCCESS;
 } /* getPacketStrInfo() */
@@ -596,7 +602,7 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
 
   if( ss==NULL || sslen==NULL || host==NULL)
 	nping_fatal(QT_3, "resolveCached(): NULL values supplied");
-	
+
   /* First we check if we have the host already cached */
   for(int i=0; i<MAX_CACHED_HOSTS && i<cached_count; i++){
     if( !strcasecmp( archive[i].hostname , host ) ){ /* Cache hit */
@@ -613,31 +619,31 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
   nping_print(DBG_4, "resolveCached(): Cache miss %d for %s\n", misses, host);
 
   if( (result=resolve(host, 0, ss, sslen, pf)) == 0 ){
-	
+
 	  /* Increment count */
 	  if( cached_count < MAX_CACHED_HOSTS )
 		cached_count++;
-		
+
       /* Store info */
 	  memset(&(archive[current_index]), 0, sizeof(cached_host_t) );
 	  strncpy(archive[current_index].hostname, host, MAX_CACHED_HOSTNAME_LEN);
 	  archive[current_index].sslen = *sslen;
 	  memcpy(&(archive[current_index].ss), ss, *sslen);
 
-	
+
 	  /* I run some tests to see what is the best approach when the cache
-	   * is full. The thing is that in Nping, we are likeley to call
+	   * is full. The thing is that in Nping, we are likely to call
 	   * this function over and over with specifying the same hosts. Deleting
 	   * the oldest entry results in 100% cache misses. I also tried to start
-	   * overwritting entries first backwards and then upwards. That showed
+	   * overwriting entries first backwards and then upwards. That showed
 	   * much better results. However, if we simply overwrite the last
 	   * cache entry over an over we get the best results. */
 	  if( current_index < MAX_CACHED_HOSTS-1 )
 		  current_index++;					
 	  return 0;
-	
 
-	
+
+
 	  ///* Watch out for the overflow. If cache is full,  */
 	  //if( cached_count == MAX_CACHED_HOSTS ){
 			//if( way%2==1 ){
@@ -660,7 +666,7 @@ int resolveCached(char *host, struct sockaddr_storage *ss, size_t *sslen, int pf
 	  //else
 		//current_index++;		
 	  //return OP_SUCCESS;
-	
+
   }else{
 		nping_warning(QT_2, "Error resolving %s\n",host);
 		return OP_FAILURE;
@@ -699,7 +705,7 @@ struct hostent *gethostbynameCached(char *host){
   nping_print(DBG_4, "gethostbynameCached(): Cache miss %d for %s", misses, host);
 
   if( (result=gethostbyname(host) ) != NULL ){
-	
+
 	  /* Increment cache entry count */
 	  if( cached_count < MAX_CACHED_HOSTS )
 		cached_count++;
@@ -709,7 +715,7 @@ struct hostent *gethostbynameCached(char *host){
        * one in its place */
       if ( current_index==MAX_CACHED_HOSTS-1 && archive[current_index].h != NULL )
         hostentfree( archive[current_index].h );
-        
+
       /* Store the hostent entry in the cache */   
 	  memset(&(archive[current_index]), 0, sizeof(gethostbynamecached_t) );
 	  strncpy(archive[current_index].hostname, host, MAX_CACHED_HOSTNAME_LEN);
@@ -723,7 +729,7 @@ struct hostent *gethostbynameCached(char *host){
 	  else{
 		  return  archive[current_index].h;
 	  }
-	
+
   }else{
     return NULL;
   }
@@ -821,7 +827,7 @@ int hostentfree(struct hostent *src){
  *  The "txt" parameter may take the special value "rand" or "random",
  *  in which case, 6 random bytes will be stored in "targetbuff".
  *  @return OP_SUCCESS on success and OP_FAILURE in case of error.
- *  Buffer targetbuff is NOT modified if "txt" does not have the propper
+ *  Buffer targetbuff is NOT modified if "txt" does not have the proper
  *  format */
 int parseMAC(const char *txt, u8 *targetbuff){
   u8 mac_data[6];
@@ -961,7 +967,7 @@ int tcppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   struct sockaddr_in6 *d6=(struct sockaddr_in6 *)dst;
   char srcipstring[128];
   char dstipstring[128];
-  
+
  assert(packet);
  assert(dstbuff);
  assert(len>=20);
@@ -972,7 +978,7 @@ int tcppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   if( detail!=LOW_DETAIL && detail!=MEDIUM_DETAIL && detail!=HIGH_DETAIL)
     detail=LOW_DETAIL;
 
-    
+
   /* Determine target IP address */
   if(src!=NULL){
     if( s4->sin_family==AF_INET ){
@@ -1059,7 +1065,7 @@ int tcppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   strncpy((char*)dstbuff, protoinfo, dstlen);
 
   return OP_SUCCESS;
-  
+
 } /* End of tcppackethdrinfo() */
 
 
@@ -1076,7 +1082,7 @@ int udppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   struct sockaddr_in6 *d6=(struct sockaddr_in6 *)dst;
   char srcipstring[128];
   char dstipstring[128];
-  
+
  assert(packet);
  assert(dstbuff);
  assert(len>=8);
@@ -1087,7 +1093,7 @@ int udppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   if( detail!=LOW_DETAIL && detail!=MEDIUM_DETAIL && detail!=HIGH_DETAIL)
     detail=LOW_DETAIL;
 
-    
+
   /* Determine target IP address */
   if(src!=NULL){
     if( s4->sin_family==AF_INET ){
@@ -1131,7 +1137,7 @@ int udppackethdrinfo(const u8 *packet, size_t len, u8 *dstbuff, size_t dstlen,
   strncpy((char*)dstbuff, protoinfo, dstlen);
 
   return OP_SUCCESS;
-  
+
 } /* End of udppackethdrinfo() */
 
 
@@ -1352,7 +1358,7 @@ u8 *getTCPheaderLocation(u8 *pkt, size_t pktLen){
   }
 
   return NULL;
-  
+
 } /* End of getTCPHeaderLocation() */
 
 
@@ -1395,7 +1401,7 @@ u16 *getSrcPortFromIPPacket(u8 *pkt, size_t pktLen){
   if((header=getTCPheaderLocation(pkt, pktLen))==NULL){
     if ((header=getUDPheaderLocation(pkt, pktLen))==NULL)
       return NULL;
-    
+
   }
   pnt=(u16*)&(header[0]);
   port= ntohs(*pnt);
@@ -1458,24 +1464,24 @@ int obtainRawSocket(){
 
   if( o.ipv6() ){
     switch( o.getMode() ){
-        
+
         case TCP:
             protocol = IPPROTO_TCP;
         break;
-        
+
         case UDP:
             protocol = IPPROTO_UDP;
         break;
-        
+
         case ICMP:
             protocol = IPPROTO_ICMPV6;
         break;
-        
+
         case ARP:
             nping_warning(QT_2,"Warning: createRawSocket() should not be called in ARP mode.");
             return 0;
         break;
-        
+
         default:
             nping_fatal(QT_3, "createRawSocket(): NpingOps::getMode() does not return a valid mode. Please report this bug.");
         break;
@@ -1487,9 +1493,9 @@ int obtainRawSocket(){
   }else{
     if ((rawipsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
         nping_fatal(QT_3,"Couldn't acquire IPv4 raw socket. Are you root?");
-        /* Tell the kernel we are including our own IP Header (call to 
-        * setsockopt passing option IP_HDRINCL)                             */
-        sethdrinclude(rawipsd);
+    /* Tell the kernel we are including our own IP Header (call to
+     * setsockopt passing option IP_HDRINCL) */
+    sethdrinclude(rawipsd);
   }
 
   /* Allow broadcast addresses */
@@ -1541,7 +1547,7 @@ int getinterfaces_inet6_linux(if6_t *ifbuf, int max_ifaces){
 
   if(ifbuf==NULL || max_ifaces<=0)
     nping_fatal(QT_3,"getinterfaces_inet6_linux() NULL values supplied");
- 
+
   /* TODO: Do we fatal() or should we just error and return OP_FAILURE? */
   if ( !file_is_readable(PATH_PROC_IFINET6) )
     nping_fatal(QT_3, "Couldn't get IPv6 interface information. File %s does not exist or you don't have read permissions.", PATH_PROC_IFINET6);
@@ -1557,7 +1563,7 @@ int getinterfaces_inet6_linux(if6_t *ifbuf, int max_ifaces){
 
     /* Check the line has the expected format ********************************/
     /* Some versions of the kernel include colons in the IPv6 address, some
-     * others dont. E.g:
+     * others don't. E.g:
      * fe80:0000:0000:0000:0333:a5ff:4444:9306 03 40 20 80 wlan0
      * fe800000000000000333a5ff44449306 03 40 20 80 wlan0
      * So what we do is to remove the colons so we can process the line
@@ -1677,7 +1683,7 @@ int getinterfaces_inet6_linux(if6_t *ifbuf, int max_ifaces){
     printf(" %02x", dev_flags); 
     printf(" %8s\n", devname);        
  */
-    
+
   } /* End of loop */
 
   /* Cleanup */
@@ -1735,7 +1741,7 @@ int getroutes_inet6_linux(route6_t *rtbuf, int max_routes){
 
   if(rtbuf==NULL || max_routes<=0)
     nping_fatal(QT_3,"getroutes_inet6_linux() NULL values supplied");
- 
+
   /* TODO: Do we fatal() or should we just error and return OP_FAILURE? */
   if ( !file_is_readable(PATH_PROC_IPV6ROUTE) )
     nping_fatal(QT_3, "Couldn't get IPv6 route information. File %s does not exist or you don't have read permissions.", PATH_PROC_IPV6ROUTE);
@@ -1751,17 +1757,17 @@ int getroutes_inet6_linux(route6_t *rtbuf, int max_routes){
 
     /* Check the line has the expected format ********************************/
     /* Some versions of the kernel include colons in the IPv6 address, some
-     * others dont. So what we do is to remove the colons so we can process
+     * others don't. So what we do is to remove the colons so we can process
      * the line no matter the format of the IPv6 addresses.
      *
      * TODO: Can interfaces with format eth0:1 appear on /proc/net/ipv6_route?
      * If they can, then we need to change the code to skip the last : */
     removecolon(buffer);
-    
+
     /* 1. Check it has the correct length.  */
     size_t min_len=0;
     min_len += 3*32; /* Three IPv6 addresses in hex */
-    min_len += 2*2;  /* Two 8bit hex values (prefiex lengths) */
+    min_len += 2*2;  /* Two 8bit hex values (prefix lengths) */
     min_len += 4*8;  /* Four 32-bit hex values */
     min_len += 1;    /* I guess one char is the min for a device len */
     min_len += 9;    /* 9 spaces */
@@ -1908,12 +1914,12 @@ int getroutes_inet6_linux(route6_t *rtbuf, int max_routes){
  *
     for(i=0; i<16; i++)
         printf("%02x", rtbuf[parsed_routes].dst_net.s6_addr[i]);
-        
+
     printf(" %02x ", rtbuf[parsed_routes].dst_prefix);
 
     for(i=0; i<16; i++)
         printf("%02x", rtbuf[parsed_routes].src_net.s6_addr[i]);
-        
+
     printf(" %02x ", rtbuf[parsed_routes].src_prefix);
 
     for(i=0; i<16; i++)
@@ -1926,7 +1932,7 @@ int getroutes_inet6_linux(route6_t *rtbuf, int max_routes){
     printf(" %8s\n", rtbuf[parsed_routes].devname);        
 */
    parsed_routes++;
-    
+
   } /* End of loop */
 
   /* Cleanup */
@@ -1958,7 +1964,7 @@ route6_t *route_dst_ipv6_linux(const struct sockaddr_storage *const dst){
   u8 zero_addr[16];                  /* Just to compare route to addr "::"     */
   memset(zero_addr, 0, 16);
   dstsin6=(struct sockaddr_in6 *)dst;
-   
+
   if(dst==NULL) return NULL;
   if(dstsin6->sin6_family!=AF_INET6) return NULL;
 
