@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2018 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -28,8 +28,7 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
@@ -54,7 +53,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: error.h 31563 2013-07-28 22:08:48Z fyodor $ */
+/* $Id: error.h 37126 2018-01-28 21:18:17Z fyodor $ */
 
 #ifndef ERROR_H
 #define ERROR_H
@@ -75,15 +74,18 @@
 #include <unistd.h>
 #endif
 
-void fatal(char *fmt, ...)
-  __attribute__ ((noreturn))
+#if defined(__GNUC__)
+#define NORETURN __attribute__((noreturn))
+#elif defined(_MSC_VER)
+#define NORETURN __declspec(noreturn)
+#else
+#define NORETURN
+#endif
+
+NORETURN void fatal(char *fmt, ...)
   __attribute__ ((format (printf, 1, 2)));
 
-void pfatal(char *fmt, ...)
-  __attribute__ ((noreturn))
+NORETURN void pfatal(char *fmt, ...)
   __attribute__ ((format (printf, 1, 2)));
 
-void gh_perror(char *err, ...);
-
 #endif /* ERROR_H */
-
