@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2013 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2018 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -30,8 +30,7 @@
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
@@ -56,7 +55,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_engines.c 31563 2013-07-28 22:08:48Z fyodor $ */
+/* $Id: nsock_engines.c 37126 2018-01-28 21:18:17Z fyodor $ */
 
 #ifdef HAVE_CONFIG_H
 #include "nsock_config.h"
@@ -64,6 +63,12 @@
 
 #include "nsock_internal.h"
 
+#if HAVE_IOCP
+  extern struct io_engine engine_iocp;
+  #define ENGINE_IOCP &engine_iocp,
+#else
+  #define ENGINE_IOCP
+#endif /* HAVE_IOCP */
 
 #if HAVE_EPOLL
   extern struct io_engine engine_epoll;
@@ -96,12 +101,34 @@ static struct io_engine *available_engines[] = {
   ENGINE_EPOLL
   ENGINE_KQUEUE
   ENGINE_POLL
+  ENGINE_IOCP
   ENGINE_SELECT
   NULL
 };
 
 static char *engine_hint;
 
+int posix_iod_connect(struct npool *nsp, int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
+  return connect(sockfd, addr, addrlen);
+}
+
+int posix_iod_read(struct npool *nsp, int sockfd, void *buf, size_t len, int flags, struct sockaddr *src_addr, socklen_t *addrlen) {
+  return recvfrom(sockfd, (char *)buf, len, flags, src_addr, addrlen);
+}
+
+int posix_iod_write(struct npool *nsp, int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen) {
+  struct sockaddr_storage *dest = (struct sockaddr_storage *)dest_addr;
+  if (dest->ss_family == AF_UNSPEC)
+    return send(sockfd, (char *)buf, len, flags);
+  else
+    return sendto(sockfd, (char *)buf, len, flags, dest_addr, addrlen);
+}
+
+struct io_operations posix_io_operations = {
+  posix_iod_connect,
+  posix_iod_read,
+  posix_iod_write
+};
 
 struct io_engine *get_io_engine(void) {
   struct io_engine *engine = NULL;
@@ -147,6 +174,9 @@ int nsock_set_default_engine(char *engine) {
 
 const char *nsock_list_engines(void) {
   return
+#if HAVE_IOCP
+  "iocp "
+#endif
 #if HAVE_EPOLL
   "epoll "
 #endif
