@@ -9,18 +9,18 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
- * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we sell    *
- * alternative licenses (contact sales@nmap.com).  Dozens of software      *
- * vendors already license Nmap technology such as host discovery, port    *
- * scanning, OS detection, version detection, and the Nmap Scripting       *
- * Engine.                                                                 *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
  * Note that the GPL places important restrictions on "derivative works",  *
  * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -62,12 +62,19 @@
  * particularly including the GPL Section 3 requirements of providing      *
  * source code and allowing free redistribution of the work as a whole.    *
  *                                                                         *
- * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
  * linked combinations including the two.                                  *
  *                                                                         *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
  * Any redistribution of Covered Software, including any derived works,    *
  * must obey and carry forward all of the terms of this license, including *
  * obeying all GPL rules and restrictions.  For example, source code of    *
@@ -86,12 +93,12 @@
  * Covered Software without special permission from the copyright holders. *
  *                                                                         *
  * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
  * information.                                                            *
  *                                                                         *
  * If you have received a written license agreement or contract for        *
@@ -107,12 +114,12 @@
  * to the dev@nmap.org mailing list for possible incorporation into the    *
  * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
@@ -125,7 +132,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc 35576 2016-01-13 20:53:39Z dmiller $ */
+/* $Id: output.cc 37126 2018-01-28 21:18:17Z fyodor $ */
 
 #include "nmap.h"
 #include "output.h"
@@ -137,7 +144,9 @@
 #include "portreasons.h"
 #include "protocols.h"
 #include "FingerPrintResults.h"
+#include "tcpip.h"
 #include "Target.h"
+#include "nmap_error.h"
 #include "utils.h"
 #include "xml.h"
 #include "nbase.h"
@@ -462,7 +471,8 @@ static std::string escape_for_screen(const std::string s) {
   for (unsigned int i = 0; i < s.size(); i++) {
     char buf[5];
     unsigned char c = s[i];
-    if (c == '\t' || c == '\r' || c == '\n' || (0x20 <= c && c <= 0x7e)) {
+    // Printable and some whitespace ok. "\r" not ok because it overwrites the line.
+    if (c == '\t' || c == '\n' || (0x20 <= c && c <= 0x7e)) {
       r += c;
     } else {
       Snprintf(buf, sizeof(buf), "\\x%02X", c);
@@ -654,8 +664,7 @@ void printportoutput(Target *currenths, PortList *plist) {
     prevstate = istate;
   }
 
-  if (prevstate != PORT_UNKNOWN)
-    log_write(LOG_PLAIN, "\n");
+  log_write(LOG_PLAIN, "\n");
 
   if (o.reason)
     print_state_summary(plist, STATE_REASON_FULL);
@@ -709,7 +718,7 @@ void printportoutput(Target *currenths, PortList *plist) {
           first = 0;
         if (o.reason) {
           if (current->reason.ttl)
-            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d", 
+            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d",
                                 port_reason_str(current->reason), current->reason.ttl);
           else
             Tbl->addItem(rowno, reasoncol, true, port_reason_str(current->reason));
@@ -774,7 +783,7 @@ void printportoutput(Target *currenths, PortList *plist) {
         Tbl->addItem(rowno, servicecol, true, serviceinfo);
         if (o.reason) {
           if (current->reason.ttl)
-            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d", 
+            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d",
                                   port_reason_str(current->reason), current->reason.ttl);
           else
             Tbl->addItem(rowno, reasoncol, true, port_reason_str(current->reason));
@@ -865,6 +874,10 @@ void printportoutput(Target *currenths, PortList *plist) {
   xml_end_tag(); /* ports */
   xml_newline();
 
+  if (o.defeat_rst_ratelimit && o.TCPScan() && plist->getStateCounts(PORT_FILTERED) > 0) {
+    log_write(LOG_PLAIN, "Some closed ports may be reported as filtered due to --defeat-rst-ratelimit\n");
+  }
+
   // Now we write the table for the user
   log_write(LOG_PLAIN, "%s", Tbl->printableTable(NULL));
   delete Tbl;
@@ -987,9 +1000,6 @@ void log_vwrite(int logt, const char *fmt, va_list ap) {
       case LOG_MACHINE:
       case LOG_SKID:
       case LOG_XML:
-        len = alloc_vsprintf(&writebuf, fmt, ap);
-        if (writebuf == NULL)
-          fatal("%s: alloc_vsprintf failed.", __func__);
         if (logtype == LOG_SKID_NOXLT)
             l = LOG_SKID;
         else
@@ -1000,17 +1010,22 @@ void log_vwrite(int logt, const char *fmt, va_list ap) {
           l >>= 1;
         }
         assert(fileidx < LOG_NUM_FILES);
-        if (o.logfd[fileidx] && len) {
-          if ((logtype & (LOG_SKID|LOG_SKID_NOXLT)) && !skid_noxlate)
-            skid_output(writebuf);
+        if (o.logfd[fileidx]) {
+          len = alloc_vsprintf(&writebuf, fmt, ap);
+          if (writebuf == NULL)
+            fatal("%s: alloc_vsprintf failed.", __func__);
+          if (len) {
+            if ((logtype & (LOG_SKID|LOG_SKID_NOXLT)) && !skid_noxlate)
+              skid_output(writebuf);
 
-          rc = fwrite(writebuf, len, 1, o.logfd[fileidx]);
-          if (rc != 1) {
-            fatal("Failed to write %d bytes of data to (logt==%d) stream. fwrite returned %d.  Quitting.", len, logtype, rc);
+            rc = fwrite(writebuf, len, 1, o.logfd[fileidx]);
+            if (rc != 1) {
+              fatal("Failed to write %d bytes of data to (logt==%d) stream. fwrite returned %d.  Quitting.", len, logtype, rc);
+            }
+            va_end(apcopy);
           }
-          va_end(apcopy);
+          free(writebuf);
         }
-        free(writebuf);
         break;
 
       default:
@@ -1097,9 +1112,9 @@ void log_flush_all() {
 }
 
 /* Open a log descriptor of the type given to the filename given.  If
-   append is nonzero, the file will be appended instead of clobbered if
+   append is true, the file will be appended instead of clobbered if
    it already exists.  If the file does not exist, it will be created */
-int log_open(int logt, int append, char *filename) {
+int log_open(int logt, bool append, char *filename) {
   int i = 0;
   if (logt <= 0 || logt > LOG_FILE_MASK)
     return -1;
@@ -1416,7 +1431,7 @@ static char *num_to_string_sigdigits(double d, int digits) {
 /* Writes a heading for a full scan report ("Nmap scan report for..."),
    including host status and DNS records. */
 void write_host_header(Target *currenths) {
-  if ((currenths->flags & HOST_UP) || o.verbose || o.resolve_all) {
+  if ((currenths->flags & HOST_UP) || o.verbose || o.always_resolve) {
     if (currenths->flags & HOST_UP) {
       log_write(LOG_PLAIN, "Nmap scan report for %s\n", currenths->NameIP());
     } else if (currenths->flags & HOST_DOWN) {
@@ -1428,17 +1443,14 @@ void write_host_header(Target *currenths) {
   }
   write_host_status(currenths);
   if (currenths->TargetName() != NULL
-      && currenths->resolved_addrs.size() > 1) {
-    const struct sockaddr_storage *hs_ss = currenths->TargetSockAddr();
+      && !currenths->unscanned_addrs.empty()) {
 
     log_write(LOG_PLAIN, "Other addresses for %s (not scanned):",
       currenths->TargetName());
-    for (std::list<struct sockaddr_storage>::const_iterator it = currenths->resolved_addrs.begin(), end = currenths->resolved_addrs.end();
+    for (std::list<struct sockaddr_storage>::const_iterator it = currenths->unscanned_addrs.begin(), end = currenths->unscanned_addrs.end();
         it != end; it++) {
       struct sockaddr_storage ss = *it;
-      if (!sockaddr_storage_equal(&ss, hs_ss)) {
-        log_write(LOG_PLAIN, " %s", inet_ntop_ez(&ss, sizeof(ss)));
-      }
+      log_write(LOG_PLAIN, " %s", inet_ntop_ez(&ss, sizeof(ss)));
     }
     log_write(LOG_PLAIN, "\n");
   }
@@ -1879,7 +1891,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "open");
     xml_attribute("proto", "tcp");
-    xml_attribute("portid", "%hu", FPR->osscan_opentcpport);
+    xml_attribute("portid", "%d", FPR->osscan_opentcpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1887,7 +1899,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "closed");
     xml_attribute("proto", "tcp");
-    xml_attribute("portid", "%hu", FPR->osscan_closedtcpport);
+    xml_attribute("portid", "%d", FPR->osscan_closedtcpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1895,7 +1907,7 @@ void printosscanoutput(Target *currenths) {
     xml_open_start_tag("portused");
     xml_attribute("state", "closed");
     xml_attribute("proto", "udp");
-    xml_attribute("portid", "%hu", FPR->osscan_closedudpport);
+    xml_attribute("portid", "%d", FPR->osscan_closedudpport);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -1921,12 +1933,22 @@ void printosscanoutput(Target *currenths) {
       for (i = 1; i < FPR->num_perfect_matches; i++)
         log_write(LOG_MACHINE, "|%s", FPR->matches[i]->OS_name);
 
+      unsigned short numprints = FPR->matches[0]->numprints;
       log_write(LOG_PLAIN, "OS details: %s", FPR->matches[0]->OS_name);
-      for (i = 1; i < FPR->num_perfect_matches; i++)
+      for (i = 1; i < FPR->num_perfect_matches; i++) {
+        numprints = MIN(numprints, FPR->matches[i]->numprints);
         log_write(LOG_PLAIN, ", %s", FPR->matches[i]->OS_name);
+      }
       log_write(LOG_PLAIN, "\n");
 
-      if (o.debugging || o.verbose > 1)
+      /* Suggest submission of an already-matching IPv6 fingerprint with
+       * decreasing probability as numprints increases, and never if the group
+       * has 5 or more prints or if the print is unsuitable. */
+      bool suggest_submission = currenths->af() == AF_INET6 && reason == NULL && rand() % 5 >= numprints;
+      if (suggest_submission)
+        log_write(LOG_NORMAL | LOG_SKID_NOXLT | LOG_STDOUT,
+            "Nmap needs more fingerprint submissions of this type. Please submit via https://nmap.org/submit/\n");
+      if (suggest_submission || o.debugging || o.verbose > 1)
         write_merged_fpr(FPR, currenths, reason == NULL, true);
     } else {
       /* No perfect matches. */
@@ -2095,7 +2117,7 @@ void printserviceinfooutput(Target *currenths) {
   Port port;
   struct serviceDeductions sd;
   int i, numhostnames = 0, numostypes = 0, numdevicetypes = 0, numcpes = 0;
-  char hostname_tbl[MAX_SERVICE_INFO_FIELDS][MAXHOSTNAMELEN];
+  char hostname_tbl[MAX_SERVICE_INFO_FIELDS][FQDN_LEN+1];
   char ostype_tbl[MAX_SERVICE_INFO_FIELDS][64];
   char devicetype_tbl[MAX_SERVICE_INFO_FIELDS][64];
   char cpe_tbl[MAX_SERVICE_INFO_FIELDS][80];
@@ -2484,7 +2506,7 @@ void print_xml_finished_open(time_t timep, const struct timeval *tv) {
   xml_attribute("timestr", "%s", mytime);
   xml_attribute("elapsed", "%.2f", o.TimeSinceStart(tv));
   xml_attribute("summary",
-    "Nmap done at %s; %d %s (%d %s up) scanned in %.2f seconds",
+    "Nmap done at %s; %u %s (%u %s up) scanned in %.2f seconds",
     mytime, o.numhosts_scanned,
     (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
     o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
@@ -2512,7 +2534,7 @@ void printfinaloutput() {
 
   if (o.numhosts_scanned == 0
 #ifndef NOLUA
-      && o.scriptupdatedb == 0
+      && !o.scriptupdatedb
 #endif
       )
     error("WARNING: No targets were specified, so 0 hosts scanned.");
@@ -2526,10 +2548,13 @@ void printfinaloutput() {
       log_write(LOG_PLAIN, "OS detection performed. Please report any incorrect results at https://nmap.org/submit/ .\n");
     else if (o.servicescan)
       log_write(LOG_PLAIN, "Service detection performed. Please report any incorrect results at https://nmap.org/submit/ .\n");
+    else if (o.udpscan && o.defeat_icmp_ratelimit)
+      log_write(LOG_PLAIN, "WARNING: Some ports marked closed|filtered may actually be open. For more accurate results, do not use --defeat-icmp-ratelimit .\n");
+
   }
 
   log_write(LOG_STDOUT | LOG_SKID,
-            "Nmap done: %d %s (%d %s up) scanned in %.2f seconds\n",
+            "Nmap done: %u %s (%u %s up) scanned in %.2f seconds\n",
             o.numhosts_scanned,
             (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
             o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
@@ -2551,7 +2576,7 @@ void printfinaloutput() {
   xml_newline();
 
   log_write(LOG_NORMAL | LOG_MACHINE,
-            "# Nmap done at %s -- %d %s (%d %s up) scanned in %.2f seconds\n",
+            "# Nmap done at %s -- %u %s (%u %s up) scanned in %.2f seconds\n",
             mytime, o.numhosts_scanned,
             (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
             o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
