@@ -5,18 +5,18 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
- * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we sell    *
- * alternative licenses (contact sales@insecure.com).  Dozens of software  *
- * vendors already license Nmap technology such as host discovery, port    *
- * scanning, OS detection, version detection, and the Nmap Scripting       *
- * Engine.                                                                 *
+ * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+ * This program is free software; you may redistribute and/or modify it    *
+ * under the terms of the GNU General Public License as published by the   *
+ * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+ * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+ * right to use, modify, and redistribute this software under certain      *
+ * conditions.  If you wish to embed Nmap technology into proprietary      *
+ * software, we sell alternative licenses (contact sales@nmap.com).        *
+ * Dozens of software vendors already license Nmap technology such as      *
+ * host discovery, port scanning, OS detection, version detection, and     *
+ * the Nmap Scripting Engine.                                              *
  *                                                                         *
  * Note that the GPL places important restrictions on "derivative works",  *
  * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -58,18 +58,25 @@
  * particularly including the GPL Section 3 requirements of providing      *
  * source code and allowing free redistribution of the work as a whole.    *
  *                                                                         *
- * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * As another special exception to the GPL terms, the Nmap Project grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
  * linked combinations including the two.                                  *
  *                                                                         *
+ * The Nmap Project has permission to redistribute Npcap, a packet         *
+ * capturing driver and library for the Microsoft Windows platform.        *
+ * Npcap is a separate work with it's own license rather than this Nmap    *
+ * license.  Since the Npcap license does not permit redistribution        *
+ * without special permission, our Nmap Windows binary packages which      *
+ * contain Npcap may not be redistributed without special permission.      *
+ *                                                                         *
  * Any redistribution of Covered Software, including any derived works,    *
  * must obey and carry forward all of the terms of this license, including *
  * obeying all GPL rules and restrictions.  For example, source code of    *
  * the whole work must be provided and free redistribution must be         *
  * allowed.  All GPL references to "this License", are to be treated as    *
- * including the special and conditions of the license text as well.       *
+ * including the terms and conditions of this license text as well.        *
  *                                                                         *
  * Because this license imposes special exceptions to the GPL, Covered     *
  * Work may not be combined (even as part of a larger work) with plain GPL *
@@ -82,34 +89,33 @@
  * Covered Software without special permission from the copyright holders. *
  *                                                                         *
  * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
+ * Nmap in other works, we are happy to help.  As mentioned above, we also *
+ * offer an alternative license to integrate Nmap into proprietary         *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * as providing support and updates.  They also fund the continued         *
+ * development of Nmap.  Please email sales@nmap.com for further           *
+ * information.                                                            *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to the dev@nmap.org mailing list for possible incorporation into the    *
  * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
+ * source code repository, it is understood (unless you specify            *
+ * otherwise) that you are offering the Nmap Project the unlimited,        *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because     *
+ * the inability to relicense code has caused devastating problems for     *
+ * other Free Software projects (such as KDE and NASM).  We also           *
  * occasionally relicense the code to third parties as discussed above.    *
  * If you wish to specify special license conditions of your               *
  * contributions, just say so when you send them.                          *
@@ -118,13 +124,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
  * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: services.cc 31563 2013-07-28 22:08:48Z fyodor $ */
+/* $Id: services.cc 37126 2018-01-28 21:18:17Z fyodor $ */
 
-#include "nmap.h"
+#include "scan_lists.h"
 #include "services.h"
 #include "NmapOps.h"
 #include "charpool.h"
@@ -199,22 +205,22 @@ static int nmap_services_init() {
     error("Unable to find nmap-services!  Resorting to /etc/services");
     strcpy(filename, "/etc/services");
 #else
-	int len, wnt = GetVersion() < 0x80000000;
+        int len, wnt = GetVersion() < 0x80000000;
     error("Unable to find nmap-services!  Resorting to /etc/services");
-	if(wnt)
-		len = GetSystemDirectory(filename, 480);	//	be safe
-	else
-		len = GetWindowsDirectory(filename, 480);	//	be safe
-	if(!len)
-		error("Get%sDirectory failed (%d) @#!#@",
-		 wnt ? "System" : "Windows", GetLastError());
-	else
-	{
-		if(wnt)
-			strcpy(filename + len, "\\drivers\\etc\\services");
-		else
-			strcpy(filename + len, "\\services");
-	}
+        if(wnt)
+                len = GetSystemDirectory(filename, 480);	//	be safe
+        else
+                len = GetWindowsDirectory(filename, 480);	//	be safe
+        if(!len)
+                error("Get%sDirectory failed (%d) @#!#@",
+                 wnt ? "System" : "Windows", GetLastError());
+        else
+        {
+                if(wnt)
+                        strcpy(filename + len, "\\drivers\\etc\\services");
+                else
+                        strcpy(filename + len, "\\services");
+        }
 #endif
   }
 
@@ -234,32 +240,32 @@ static int nmap_services_init() {
       continue;
 
     res = sscanf(line, "%127s %hu/%15s %31s", servicename, &portno, proto, ratio_str);
-    
+
     if (res == 3) {
       ratio = 0;
     } else if (res == 4) {
       if (strchr(ratio_str, '/')) {
-	res = sscanf(ratio_str, "%d/%d", &ratio_n, &ratio_d);
-	if (res != 2)
-	  fatal("%s:%d contains invalid port ratio string: %s", filename, lineno, ratio_str);
+        res = sscanf(ratio_str, "%d/%d", &ratio_n, &ratio_d);
+        if (res != 2)
+          fatal("%s:%d contains invalid port ratio string: %s", filename, lineno, ratio_str);
 
-	if (ratio_n < 0 || ratio_d < 0)
-	  fatal("%s:%d contains an invalid negative value", filename, lineno);
-	
-	if (ratio_n > ratio_d)
-	  fatal("%s:%d has a ratio %g. All ratios must be < 1", filename, lineno, (double)ratio_n/ratio_d);
-	
-	if (ratio_d == 0)
-	  fatal("%s:%d has a ratio denominator of 0 causing a division by 0 error", filename, lineno);
-	
-	ratio = (double)ratio_n / ratio_d;
-	ratio_format = 1;
+        if (ratio_n < 0 || ratio_d < 0)
+          fatal("%s:%d contains an invalid negative value", filename, lineno);
+
+        if (ratio_n > ratio_d)
+          fatal("%s:%d has a ratio %g. All ratios must be < 1", filename, lineno, (double)ratio_n/ratio_d);
+
+        if (ratio_d == 0)
+          fatal("%s:%d has a ratio denominator of 0 causing a division by 0 error", filename, lineno);
+
+        ratio = (double)ratio_n / ratio_d;
+        ratio_format = 1;
       } else if (strncmp(ratio_str, "0.", 2) == 0) {
-	/* We assume the ratio is in floating point notation already */
-	ratio = strtod(ratio_str, NULL);
-	ratio_format = 1;
+        /* We assume the ratio is in floating point notation already */
+        ratio = strtod(ratio_str, NULL);
+        ratio_format = 1;
       } else {
-	ratio = 0;
+        ratio = 0;
       }
     } else {
       continue;
@@ -292,7 +298,7 @@ static int nmap_services_init() {
       /* possibly misplaced comment, but who cares? */
     } else {
       if (o.debugging)
-	error("Unknown protocol (%s) on line %d of services file %s.", proto, lineno, filename);
+        error("Unknown protocol (%s) on line %d of services file %s.", proto, lineno, filename);
       continue;
     }
 
@@ -325,7 +331,7 @@ void free_services() {
   services_initialized = 0;
 }
 
-  
+
 /* Adds ports whose names match mask and one or more protocols
  * specified by range_type to porttbl. Increases the respective
  * protocol counts in ports.
@@ -338,7 +344,7 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
 
   if (!services_initialized && nmap_services_init() == -1)
     fatal("%s: Couldn't get port numbers", __func__);
-  
+
   for (i = service_table.begin(); i != service_table.end(); i++) {
     service_node& current = i->second;
     if (wildtest(mask, current.s_name)) {
@@ -424,10 +430,13 @@ static bool is_port_member(const struct scan_lists *ptsdata, const struct servic
 // If level is 1 or above, we treat it as a "top ports" directive
 // and return the N highest ratio ports (where N==level).
 //
+// If the fourth parameter is not NULL, then the specified ports
+// are excluded first and only then are the top N ports taken
+//
 // This function doesn't support IP protocol scan so only call this
 // function if o.TCPScan() || o.UDPScan() || o.SCTPScan()
 
-void gettoppts(double level, char *portlist, struct scan_lists * ports) {
+void gettoppts(double level, char *portlist, struct scan_lists * ports, char *exclude_ports) {
   int ti=0, ui=0, si=0;
   struct scan_lists ptsdata = { 0 };
   bool ptsdata_initialized = false;
@@ -466,7 +475,14 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
   if (portlist){
     getpts(portlist, &ptsdata);
     ptsdata_initialized = true;
+  } else if (exclude_ports) {
+    getpts("-", &ptsdata);
+    ptsdata_initialized = true;
   }
+
+  if (ptsdata_initialized && exclude_ports)
+    removepts(exclude_ports, &ptsdata);
+
   if (level < 1) {
     for (i = services_by_ratio.begin(); i != services_by_ratio.end(); i++) {
       current = &(*i);
@@ -566,4 +582,3 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
   else if (o.debugging && level >= 1)
     log_write(LOG_STDOUT, "PORTS: Using top %d ports found open (TCP:%d, UDP:%d, SCTP:%d)\n", (int) level, ports->tcp_count, ports->udp_count, ports->sctp_count);
 }
-
