@@ -2,89 +2,126 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
 # * misunderstandings, we interpret that term as broadly as copyright law   *
 # * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * o Integrates source code from Covered Software.                         *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * linked combinations including the two.                                  *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the terms and conditions of this license text as well.        *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
+# * information.                                                            *
+# *                                                                         *
+# * If you have received a written license agreement or contract for        *
+# * Covered Software stating terms other than these, you may choose to use  *
+# * and redistribute Covered Software under those terms instead of these.   *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -95,6 +132,7 @@ import gobject
 from radialnet.bestwidgets.buttons import *
 from radialnet.gui.SaveDialog import SaveDialog
 from radialnet.gui.Dialogs import AboutDialog
+from radialnet.gui.LegendWindow import LegendWindow
 from radialnet.gui.HostsViewer import HostsViewer
 from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog
 
@@ -105,7 +143,6 @@ HIDE = False
 REFRESH_RATE = 500
 
 
-
 class ToolsMenu(gtk.Menu):
     """
     """
@@ -118,7 +155,6 @@ class ToolsMenu(gtk.Menu):
 
         self.__create_items()
 
-
     def __create_items(self):
         """
         """
@@ -132,7 +168,6 @@ class ToolsMenu(gtk.Menu):
 
         self.__hosts.show_all()
 
-
     def __hosts_viewer_callback(self, widget):
         """
         """
@@ -140,20 +175,17 @@ class ToolsMenu(gtk.Menu):
         window.show_all()
         window.set_keep_above(True)
 
-
     def enable_dependents(self):
         """
         """
         self.__hosts.set_sensitive(True)
 
-
     def disable_dependents(self):
         """
         """
         self.__hosts.set_sensitive(False)
 
 
-
 class Toolbar(gtk.HBox):
     """
     """
@@ -182,7 +214,6 @@ class Toolbar(gtk.HBox):
 
         self.__create_widgets()
 
-
     def __create_widgets(self):
         """
         """
@@ -202,7 +233,8 @@ class Toolbar(gtk.HBox):
         self.__hosts_button = BWStockButton(gtk.STOCK_INDEX, _("Hosts Viewer"))
         self.__hosts_button.connect("clicked", self.__hosts_viewer_callback)
 
-        self.__control = BWToggleStockButton(gtk.STOCK_PROPERTIES, _("Controls"))
+        self.__control = BWToggleStockButton(
+                gtk.STOCK_PROPERTIES, _("Controls"))
         self.__control.connect('clicked', self.__control_callback)
         self.__control.set_active(False)
 
@@ -210,6 +242,9 @@ class Toolbar(gtk.HBox):
         self.__fisheye.connect('clicked', self.__fisheye_callback)
         self.__fisheye.set_active(False)
 
+        self.__legend_button = BWStockButton(gtk.STOCK_INDEX, _("Legend"))
+        self.__legend_button.connect('clicked', self.__legend_callback)
+
         #self.__fullscreen = gtk.ToggleToolButton(gtk.STOCK_FULLSCREEN)
         #self.__fullscreen.set_label(_('Fullscreen'))
         #self.__fullscreen.set_is_important(True)
@@ -241,7 +276,7 @@ class Toolbar(gtk.HBox):
         self.pack_start(self.__fisheye, False)
         self.pack_start(self.__control, False)
         self.pack_end(self.__save_button, False)
-
+        self.pack_end(self.__legend_button, False)
 
     def disable_controls(self):
         """
@@ -249,24 +284,23 @@ class Toolbar(gtk.HBox):
         self.__control.set_sensitive(False)
         self.__fisheye.set_sensitive(False)
         self.__hosts_button.set_sensitive(False)
+        self.__legend_button.set_sensitive(False)
         #self.__tools_menu.disable_dependents()
 
-
     def enable_controls(self):
         """
         """
         self.__control.set_sensitive(True)
         self.__fisheye.set_sensitive(True)
         self.__hosts_button.set_sensitive(True)
+        self.__legend_button.set_sensitive(True)
         #self.__tools_menu.enable_dependents()
 
-
     def __tools_callback(self, widget):
         """
         """
         self.__tools_menu.popup(None, None, None, 1, 0)
 
-
     def __hosts_viewer_callback(self, widget):
         """
         """
@@ -274,7 +308,6 @@ class Toolbar(gtk.HBox):
         window.show_all()
         window.set_keep_above(True)
 
-
     def __save_image_callback(self, widget):
         """
         """
@@ -299,7 +332,6 @@ class Toolbar(gtk.HBox):
 
         self.__save_chooser.hide()
 
-
     def __control_callback(self, widget=None):
         """
         """
@@ -309,7 +341,6 @@ class Toolbar(gtk.HBox):
         else:
             self.__control_widget.hide()
 
-
     def __fisheye_callback(self, widget=None):
         """
         """
@@ -325,6 +356,11 @@ class Toolbar(gtk.HBox):
                 self.__fisheye_widget.deactive_fisheye()
                 self.__fisheye_widget.hide()
 
+    def __legend_callback(self, widget):
+        """
+        """
+        self.__legend_window = LegendWindow()
+        self.__legend_window.show_all()
 
     def __about_callback(self, widget):
         """
@@ -332,7 +368,6 @@ class Toolbar(gtk.HBox):
         self.__about_dialog = AboutDialog()
         self.__about_dialog.show_all()
 
-
     def __fullscreen_callback(self, widget=None):
         """
         """
@@ -341,4 +376,3 @@ class Toolbar(gtk.HBox):
 
         else:
             self.__window.unfullscreen()
-
