@@ -3,18 +3,18 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@insecure.com).  Dozens of software  *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2018 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derivative works",  *
 # * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -56,18 +56,25 @@
 # * particularly including the GPL Section 3 requirements of providing      *
 # * source code and allowing free redistribution of the work as a whole.    *
 # *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
 # * linked combinations including the two.                                  *
 # *                                                                         *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
 # * Any redistribution of Covered Software, including any derived works,    *
 # * must obey and carry forward all of the terms of this license, including *
 # * obeying all GPL rules and restrictions.  For example, source code of    *
 # * the whole work must be provided and free redistribution must be         *
 # * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the special and conditions of the license text as well.       *
+# * including the terms and conditions of this license text as well.        *
 # *                                                                         *
 # * Because this license imposes special exceptions to the GPL, Covered     *
 # * Work may not be combined (even as part of a larger work) with plain GPL *
@@ -80,34 +87,33 @@
 # * Covered Software without special permission from the copyright holders. *
 # *                                                                         *
 # * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
+# * information.                                                            *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * If you have received a written license agreement or contract for        *
+# * Covered Software stating terms other than these, you may choose to use  *
+# * and redistribute Covered Software under those terms instead of these.   *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to the dev@nmap.org mailing list for possible incorporation into the    *
 # * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
@@ -116,7 +122,7 @@
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 # * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
 # * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -175,15 +181,15 @@ if isinstance(umitdb, str):
         fs_enc = "UTF-8"
     umitdb = umitdb.decode(fs_enc)
 
-# pyslite 2.4.0 doesn't handle a unicode database name, though earlier and later
-# versions do. Encode to UTF-8 as pysqlite would do internally anyway.
+# pysqlite 2.4.0 doesn't handle a unicode database name, though earlier and
+# later versions do. Encode to UTF-8 as pysqlite would do internally anyway.
 umitdb = umitdb.encode("UTF-8")
 
 connection = sqlite.connect(umitdb)
 
 # By default pysqlite will raise an OperationalError when trying to return a
 # TEXT data type that is not UTF-8 (it always tries to decode text in order to
-# return a unicdoe object). We store XML in the database, which may have a
+# return a unicode object). We store XML in the database, which may have a
 # different encoding, so instruct pysqlite to return a plain str for TEXT data
 # types, and not to attempt any decoding.
 try:
@@ -192,6 +198,7 @@ except AttributeError:
     # However, text_factory is available only in pysqlite 2.1.0 and later.
     pass
 
+
 class Table(object):
     def __init__(self, table_name):
         self.table_name = table_name
@@ -203,9 +210,11 @@ class Table(object):
         if self.__getattribute__("_%s" % item_name):
             return self.__getattribute__("_%s" % item_name)
 
-        sql = "SELECT %s FROM %s WHERE %s_id = %s" % (item_name, self.table_name,
-                                                      self.table_name,
-                                                      self.__getattribute__(self.table_id))
+        sql = "SELECT %s FROM %s WHERE %s_id = %s" % (
+                item_name,
+                self.table_name,
+                self.table_name,
+                self.__getattribute__(self.table_id))
 
         self.cursor.execute(sql)
 
@@ -216,9 +225,11 @@ class Table(object):
         if item_value == self.__getattribute__("_%s" % item_name):
             return None
 
-        sql = "UPDATE %s SET %s = ? WHERE %s_id = %s" % (self.table_name, item_name,
-                                                         self.table_name,
-                                                         self.__getattribute__(self.table_id))
+        sql = "UPDATE %s SET %s = ? WHERE %s_id = %s" % (
+                self.table_name,
+                item_name,
+                self.table_name,
+                self.__getattribute__(self.table_id))
         self.cursor.execute(sql, (item_value,))
         connection.commit()
         self.__setattr__("_%s" % item_name, item_value)
@@ -247,31 +258,30 @@ class Table(object):
         self.cursor.execute(sql)
         return self.cursor.fetchall()[0][0]
 
+
 class UmitDB(object):
     def __init__(self):
         self.cursor = connection.cursor()
 
     def create_db(self):
-        drop_string = ("DROP TABLE scans;",)
+        drop_string = "DROP TABLE scans;"
 
         try:
-            for d in drop_string:
-                self.cursor.execute(d)
+            self.cursor.execute(drop_string)
         except:
             connection.rollback()
         else:
             connection.commit()
 
+        creation_string = """CREATE TABLE scans (
+            scans_id INTEGER PRIMARY KEY AUTOINCREMENT,
+            scan_name TEXT,
+            nmap_xml_output TEXT,
+            digest TEXT,
+            date INTEGER)"""
 
-        creation_string = ("""CREATE TABLE scans (scans_id INTEGER PRIMARY KEY AUTOINCREMENT,
-                                                  scan_name TEXT,
-                                                  nmap_xml_output TEXT,
-                                                  digest TEXT,
-                                                  date INTEGER)""",)
-
-        for c in creation_string:
-            self.cursor.execute(c)
-            connection.commit()
+        self.cursor.execute(creation_string)
+        connection.commit()
 
     def add_scan(self, **kargs):
         return Scans(**kargs)
@@ -288,12 +298,14 @@ class UmitDB(object):
 
     def cleanup(self, save_time):
         log.debug(">>> Cleaning up data base.")
-        log.debug(">>> Removing results olders than %s seconds" % save_time)
-        self.cursor.execute("SELECT scans_id FROM scans WHERE date < ?", (time() - save_time,))
+        log.debug(">>> Removing results older than %s seconds" % save_time)
+        self.cursor.execute("SELECT scans_id FROM scans WHERE date < ?",
+                (time() - save_time,))
 
         for sid in [sid[0] for sid in self.cursor.fetchall()]:
             log.debug(">>> Removing results with scans_id %s" % sid)
-            self.cursor.execute("DELETE FROM scans WHERE scans_id = ?", (sid, ))
+            self.cursor.execute("DELETE FROM scans WHERE scans_id = ?",
+                    (sid, ))
         else:
             connection.commit()
             log.debug(">>> Data base successfully cleaned up!")
@@ -310,25 +322,30 @@ class Scans(Table, object):
 
             for k in kargs.keys():
                 if k not in fields:
-                    raise Exception("Wrong table field passed to creation method. '%s'" % k)
+                    raise Exception(
+                            "Wrong table field passed to creation method. "
+                            "'%s'" % k)
 
-            if "nmap_xml_output" not in kargs.keys() or not kargs["nmap_xml_output"]:
+            if ("nmap_xml_output" not in kargs.keys() or
+                    not kargs["nmap_xml_output"]):
                 raise Exception("Can't save result without xml output")
 
-            if not self.verify_digest(md5(kargs["nmap_xml_output"]).hexdigest()):
+            if not self.verify_digest(
+                    md5(kargs["nmap_xml_output"]).hexdigest()):
                 raise Exception("XML output registered already!")
 
             self.scans_id = self.insert(**kargs)
 
     def verify_digest(self, digest):
-        self.cursor.execute("SELECT scans_id FROM scans WHERE digest = ?", (digest, ))
+        self.cursor.execute(
+                "SELECT scans_id FROM scans WHERE digest = ?", (digest, ))
         result = self.cursor.fetchall()
         if result:
             return False
         return True
 
     def add_host(self, **kargs):
-        kargs.update({self.table_id:self.scans_id})
+        kargs.update({self.table_id: self.scans_id})
         return Hosts(**kargs)
 
     def get_hosts(self):
